#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2017-2018 Stephen Gallagher
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

from os import path
import sys
import logging

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
    from gi.repository import GLib
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase
import random


class TestModuleIndexMerger(TestBase):
    def test_constructors(self):
        merger = Modulemd.ModuleIndexMerger()
        self.assertIsNotNone(merger)

        merger = Modulemd.ModuleIndexMerger.new()
        self.assertIsNotNone(merger)

    def test_deduplicate(self):
        merger = Modulemd.ModuleIndexMerger()
        idx1 = Modulemd.ModuleIndex()
        idx2 = Modulemd.ModuleIndex()

        res, failures = idx1.update_from_file(
            path.join(self.test_data_path, "long-valid.yaml"), True
        )
        self.assertTrue(res)
        self.assertEqual(len(failures), 0)

        res, failures = idx2.update_from_file(
            path.join(self.test_data_path, "long-valid.yaml"), True
        )
        self.assertTrue(res)
        self.assertEqual(len(failures), 0)

        # Save the original document for comparing later
        baseline = idx1.dump_to_string()

        # Add the same index twice
        merger.associate_index(idx1, 0)
        merger.associate_index(idx2, 0)
        deduplicated_idx = merger.resolve()
        final = deduplicated_idx.dump_to_string()

        # Verify that it outputs the same content as the baseline
        self.assertEqual(baseline, final)

    def test_merger(self):
        # Get a set of objects in a ModuleIndex
        base_index = Modulemd.ModuleIndex()
        base_index.update_from_file(
            path.join(self.test_data_path, "merging-base.yaml"), True
        )

        # Baseline
        httpd_defaults = base_index.get_module("httpd").get_defaults()
        self.assertIsNotNone(httpd_defaults)
        self.assertEqual(httpd_defaults.get_default_stream(), "2.2")
        httpd_profile_streams = (
            httpd_defaults.get_streams_with_default_profiles()
        )
        self.assertEqual(len(httpd_profile_streams), 2)
        self.assertTrue("2.2" in httpd_profile_streams)
        self.assertTrue("2.8" in httpd_profile_streams)
        self.assertEqual(
            len(httpd_defaults.get_default_profiles_for_stream("2.2")), 2
        )
        self.assertTrue(
            "client" in httpd_defaults.get_default_profiles_for_stream("2.2")
        )
        self.assertTrue(
            "server" in httpd_defaults.get_default_profiles_for_stream("2.2")
        )
        self.assertTrue(
            "notreal" in httpd_defaults.get_default_profiles_for_stream("2.8")
        )

        self.assertEqual(
            httpd_defaults.get_default_stream("workstation"), "2.4"
        )
        httpd_profile_streams = (
            httpd_defaults.get_streams_with_default_profiles("workstation")
        )
        self.assertEqual(len(httpd_profile_streams), 2)
        self.assertTrue("2.4" in httpd_profile_streams)
        self.assertTrue("2.6" in httpd_profile_streams)

        self.assertEqual(
            len(
                httpd_defaults.get_default_profiles_for_stream(
                    "2.4", "workstation"
                )
            ),
            1,
        )
        self.assertEqual(
            len(
                httpd_defaults.get_default_profiles_for_stream(
                    "2.6", "workstation"
                )
            ),
            3,
        )

        # Get another set of objects that will override the default stream for
        # nodejs
        override_nodejs_index = Modulemd.ModuleIndex()
        override_nodejs_index.update_from_file(
            path.join(self.test_data_path, "overriding-nodejs.yaml"), True
        )

        # Test that adding both of these at the same priority level results in
        # the no default stream
        merger = Modulemd.ModuleIndexMerger()
        merger.associate_index(base_index, 0)
        merger.associate_index(override_nodejs_index, 0)

        merged_index = merger.resolve()
        self.assertIsNotNone(merged_index)

        nodejs = merged_index.get_module("nodejs")
        self.assertIsNotNone(nodejs)

        nodejs_defaults = nodejs.get_defaults()
        self.assertIsNotNone(nodejs_defaults)
        self.assertIsNone(nodejs_defaults.get_default_stream())

        # Get another set of objects that will override the above
        override_index = Modulemd.ModuleIndex()
        override_index.update_from_file(
            path.join(self.test_data_path, "overriding.yaml"), True
        )

        # Test that override_index at a higher priority level succeeds
        # Test that adding both of these at the same priority level fails
        # with a merge conflict.
        # Use randomly-selected high and low values to make sure we don't have
        # sorting issues.
        merger = Modulemd.ModuleIndexMerger()
        random_low = random.randint(1, 100)
        random_high = random.randint(101, 999)
        print(
            "Low priority: %d, High priority: %d" % (random_low, random_high)
        )
        merger.associate_index(base_index, random_low)
        merger.associate_index(override_index, random_high)

        merged_index = merger.resolve()
        self.assertIsNotNone(merged_index)

        # Validate merged results

        # HTTPD
        httpd_defaults = merged_index.get_module("httpd").get_defaults()
        self.assertIsNotNone(httpd_defaults)
        self.assertEqual(httpd_defaults.get_default_stream(), "2.4")
        httpd_profile_streams = (
            httpd_defaults.get_streams_with_default_profiles()
        )
        self.assertEqual(len(httpd_profile_streams), 2)
        self.assertTrue("2.2" in httpd_profile_streams)
        self.assertTrue("2.4" in httpd_profile_streams)
        self.assertEqual(
            len(httpd_defaults.get_default_profiles_for_stream("2.2")), 2
        )
        self.assertTrue(
            "client" in httpd_defaults.get_default_profiles_for_stream("2.2")
        )
        self.assertTrue(
            "server" in httpd_defaults.get_default_profiles_for_stream("2.2")
        )
        self.assertTrue(
            "client" in httpd_defaults.get_default_profiles_for_stream("2.4")
        )
        self.assertTrue(
            "server" in httpd_defaults.get_default_profiles_for_stream("2.4")
        )

        self.assertEqual(
            httpd_defaults.get_default_stream("workstation"), "2.8"
        )
        httpd_profile_streams = (
            httpd_defaults.get_streams_with_default_profiles("workstation")
        )
        self.assertEqual(len(httpd_profile_streams), 3)
        self.assertTrue("2.4" in httpd_profile_streams)
        self.assertTrue("2.6" in httpd_profile_streams)
        self.assertTrue("2.8" in httpd_profile_streams)
        self.assertEqual(
            len(
                httpd_defaults.get_default_profiles_for_stream(
                    "2.4", "workstation"
                )
            ),
            1,
        )
        self.assertEqual(
            len(
                httpd_defaults.get_default_profiles_for_stream(
                    "2.6", "workstation"
                )
            ),
            3,
        )
        self.assertEqual(
            len(
                httpd_defaults.get_default_profiles_for_stream(
                    "2.8", "workstation"
                )
            ),
            4,
        )

    def test_merger_with_real_world_data(self):
        fedora_index = Modulemd.ModuleIndex()
        fedora_index.update_from_file(
            path.join(self.test_data_path, "f29.yaml"), True
        )

        updates_index = Modulemd.ModuleIndex()
        updates_index.update_from_file(
            path.join(self.test_data_path, "f29-updates.yaml"), True
        )

        merger = Modulemd.ModuleIndexMerger()
        merger.associate_index(fedora_index, 0)
        merger.associate_index(updates_index, 0)
        merged_index = merger.resolve()

        self.assertIsNotNone(merged_index)

    def test_merger_with_modified(self):
        pass

    def test_strict_default_streams(self):
        merger = Modulemd.ModuleIndexMerger.new()

        for stream in ("27", "38"):
            default = """
---
document: modulemd-defaults
version: 1
data:
    module: python
    stream: %s
...
""" % (
                stream
            )

            index = Modulemd.ModuleIndex()
            index.update_from_string(default, strict=True)
            merger.associate_index(index, 0)

        with self.assertRaisesRegex(
            gi.repository.GLib.GError,
            "Default stream mismatch in module python",
        ):
            merger.resolve_ext(True)

    def test_merge_add_only(self):
        base_idx = Modulemd.ModuleIndex()
        self.assertTrue(
            base_idx.update_from_file(
                path.join(self.test_data_path, "merger", "base.yaml"), True
            )
        )

        add_only_idx = Modulemd.ModuleIndex()
        self.assertTrue(
            add_only_idx.update_from_file(
                path.join(self.test_data_path, "merger", "add_only.yaml"), True
            )
        )

        merger = Modulemd.ModuleIndexMerger()
        merger.associate_index(base_idx, 0)
        merger.associate_index(add_only_idx, 0)

        merged_idx = merger.resolve()
        self.assertIsNotNone(merged_idx)

        httpd = merged_idx.get_module("httpd")
        self.assertIsNotNone(httpd)
        httpd_defs = httpd.get_defaults()

        self.assertEqual(httpd_defs.get_default_stream(), "2.8")
        expected_profile_defs = {
            "2.2": set(["client", "server"]),
            "2.8": set(["notreal"]),
            "2.10": set(["notreal"]),
        }

        for stream in expected_profile_defs.keys():
            self.assertEqual(
                set(httpd_defs.get_default_profiles_for_stream(stream)),
                expected_profile_defs[stream],
            )

        self.assertEqual(httpd_defs.get_default_stream("workstation"), "2.4")

    def test_merge_add_conflicting_stream(self):
        base_idx = Modulemd.ModuleIndex()
        self.assertTrue(
            base_idx.update_from_file(
                path.join(self.test_data_path, "merger", "base.yaml"), True
            )
        )

        add_only_idx = Modulemd.ModuleIndex()
        self.assertTrue(
            add_only_idx.update_from_file(
                path.join(
                    self.test_data_path,
                    "merger",
                    "add_conflicting_stream.yaml",
                ),
                True,
            )
        )

        merger = Modulemd.ModuleIndexMerger()
        merger.associate_index(base_idx, 0)
        merger.associate_index(add_only_idx, 0)

        merged_idx = merger.resolve()
        self.assertIsNotNone(merged_idx)

        psql = merged_idx.get_module("postgresql")
        self.assertIsNotNone(psql)

        psql_defs = psql.get_defaults()
        self.assertIsNotNone(psql_defs)

        self.assertIsNone(psql_defs.get_default_stream())

        expected_profile_defs = {
            "8.1": set(["client", "server", "foo"]),
            "8.2": set(["client", "server", "foo"]),
        }

        for stream in expected_profile_defs.keys():
            self.assertEqual(
                set(psql_defs.get_default_profiles_for_stream(stream)),
                expected_profile_defs[stream],
            )

    def test_merge_add_conflicting_stream_and_profile_modified(self):
        base_idx = Modulemd.ModuleIndex()
        self.assertTrue(
            base_idx.update_from_file(
                path.join(self.test_data_path, "merger", "base.yaml"), True
            )
        )

        add_conflicting_idx = Modulemd.ModuleIndex()
        self.assertTrue(
            add_conflicting_idx.update_from_file(
                path.join(
                    self.test_data_path,
                    "merger",
                    "add_conflicting_stream_and_profile_modified.yaml",
                ),
                True,
            )
        )

        merger = Modulemd.ModuleIndexMerger()
        merger.associate_index(base_idx, 0)
        merger.associate_index(add_conflicting_idx, 0)

        merged_idx = merger.resolve()
        self.assertIsNotNone(merged_idx)

        psql = merged_idx.get_module("postgresql")
        self.assertIsNotNone(psql)

        psql_defs = psql.get_defaults()
        self.assertIsNotNone(psql_defs)

        self.assertEqual(psql_defs.get_default_stream(), "8.2")

        expected_profile_defs = {
            "8.1": set(["client", "server"]),
            "8.2": set(["client", "server", "foo"]),
            "8.3": set(["client", "server"]),
        }

        for stream in expected_profile_defs:
            self.assertEqual(
                set(psql_defs.get_default_profiles_for_stream(stream)),
                expected_profile_defs[stream],
            )

    def test_merge_incompatible_streams(self):
        # This test verifies that if we encounter two streams with the same
        # NSVCA, but different content, we only retain one of them.
        # Note: the specification of the merger states that the behavior is
        # undefined, so we will only validate that the merge completes and
        # it only contains a single stream
        merger = Modulemd.ModuleIndexMerger.new()

        base_idx = Modulemd.ModuleIndex()
        self.assertTrue(
            base_idx.update_from_file(
                path.join(self.test_data_path, "merger", "conflict_base.yaml"),
                True,
            )
        )
        merger.associate_index(base_idx, 0)

        bad_idx = Modulemd.ModuleIndex()
        self.assertTrue(
            bad_idx.update_from_file(
                path.join(self.test_data_path, "merger", "conflict_base.yaml"),
                True,
            )
        )
        # Modify the stream to have a different summary
        stream = bad_idx.search_streams_by_nsvca_glob()[0]
        stream.set_summary("Invalid summary, should not merge successfully")
        merger.associate_index(bad_idx, 0)

        merged_idx = merger.resolve()
        all_streams = merged_idx.search_streams_by_nsvca_glob()
        self.assertEqual(len(all_streams), 1)
        self.assertEqual(all_streams[0].props.module_name, "nodejs")

    def test_merge_order_independent_obsoletes(self):
        # This test verifies that if we encounter two obsoletes with the same
        # module, stream and context we associate the newer one with exisitng
        # stream
        idx = Modulemd.ModuleIndex.new()
        stream = Modulemd.ModuleStream.new(2, "nodejs", "8.0")
        stream.props.context = "42"
        res = idx.add_module_stream(stream)

        newer_idx = Modulemd.ModuleIndex()
        self.assertTrue(
            newer_idx.update_from_file(
                path.join(
                    self.test_data_path,
                    "merger",
                    "newer_obsoletes_resseted.yaml",
                ),
                True,
            )
        )

        base_idx = Modulemd.ModuleIndex()
        self.assertTrue(
            base_idx.update_from_file(
                path.join(
                    self.test_data_path, "merger", "base_obsoletes.yaml"
                ),
                True,
            )
        )

        merger = Modulemd.ModuleIndexMerger.new()
        merger.associate_index(idx, 0)
        merger.associate_index(newer_idx, 0)
        merger.associate_index(base_idx, 0)
        merged_idx = merger.resolve()

        module = merged_idx.get_module("nodejs")
        self.assertEqual(len(module.get_obsoletes()), 2)

        all_streams = merged_idx.search_streams_by_nsvca_glob()
        self.assertEqual(len(all_streams), 1)
        self.assertEqual(all_streams[0].props.module_name, "nodejs")

        obsoletes = all_streams[0].get_obsoletes_resolved()
        # obsoletes is None because it has reset: true
        self.assertEqual(obsoletes, None)

        # Merge the indexes in reverse order
        merger = Modulemd.ModuleIndexMerger.new()
        merger.associate_index(idx, 0)
        merger.associate_index(base_idx, 0)
        merger.associate_index(newer_idx, 0)
        merged_idx = merger.resolve()

        module = merged_idx.get_module("nodejs")
        self.assertEqual(len(module.get_obsoletes()), 2)

        all_streams = merged_idx.search_streams_by_nsvca_glob()
        self.assertEqual(len(all_streams), 1)
        self.assertEqual(all_streams[0].props.module_name, "nodejs")

        obsoletes = all_streams[0].get_obsoletes_resolved()
        # obsoletes is None because it has reset: true
        self.assertEqual(obsoletes, None)


if __name__ == "__main__":
    unittest.main()



# This file is part of libmodulemd
# Copyright (C) 2016 Red Hat, Inc.
# Copyright (C) 2017-2018 Stephen Gallagher
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

from ..module import get_introspection_module
from ..overrides import override

import functools

from six import text_type
from gi.repository import GLib

import datetime

Modulemd = get_introspection_module("Modulemd")


__all__ = []


class ModulemdUtil(object):
    @staticmethod
    def strip_gtype(method):
        @functools.wraps(method)
        def wrapped(*args, **kwargs):
            ret = method(*args, **kwargs)
            if len(ret) == 2:
                return ret[1]
            else:
                return ret[1:]

        return wrapped

    @staticmethod
    def variant_str(s):
        """Converts a string to a GLib.Variant"""
        if not isinstance(s, str):
            raise TypeError("Only strings are supported for scalars")

        return GLib.Variant.new_string(s)

    @staticmethod
    def variant_bool(b):
        """Converts a boolean to a GLib.Varant"""
        if not isinstance(b, bool):
            raise TypeError("Only booleans are supported")

        return GLib.Variant.new_boolean(b)

    @staticmethod
    def variant_list(vl):
        """Converts a list to a GLib.Variant"""

        # If this is a zero-length array, handle it specially
        if len(vl) == 0:
            return GLib.Variant.new_array(GLib.VariantType("v"))

        # Build the array from each entry
        builder = GLib.VariantBuilder(GLib.VariantType("a*"))
        for item in vl:
            if item is None:
                item = ""
            builder.add_value(ModulemdUtil.python_to_variant(item))

        return builder.end()

    @staticmethod
    def variant_dict(d):
        """Converts a dictionary to a dictionary of GLib.Variant"""
        if not isinstance(d, dict):
            raise TypeError("Only dictionaries are supported for mappings")

        vdict = GLib.VariantDict()

        for k, v in d.items():
            if v is None:
                v = ""
            vdict.insert_value(k, ModulemdUtil.python_to_variant(v))

        return vdict.end()

    @staticmethod
    def python_to_variant(obj):

        if isinstance(obj, str):
            return ModulemdUtil.variant_str(obj)

        elif isinstance(obj, text_type):
            return ModulemdUtil.variant_str(obj.encode("utf-8"))

        elif isinstance(obj, bool):
            return ModulemdUtil.variant_bool(obj)

        elif isinstance(obj, list):
            return ModulemdUtil.variant_list(obj)

        elif isinstance(obj, dict):
            return ModulemdUtil.variant_dict(obj)

        else:
            raise TypeError("Cannot convert unknown type")


if float(Modulemd._version) >= 2:

    if hasattr(Modulemd, "read_packager_file"):
        read_packager_file = ModulemdUtil.strip_gtype(
            Modulemd.read_packager_file
        )
        __all__.append("read_packager_file")

    if hasattr(Modulemd, "read_packager_string"):
        read_packager_string = ModulemdUtil.strip_gtype(
            Modulemd.read_packager_string
        )
        __all__.append("read_packager_string")

    if hasattr(Modulemd.ModuleIndex, "add_known_stream"):

        class ModuleIndex(Modulemd.ModuleIndex):
            def set_known_streams(self, known_streams):
                for module, streams in known_streams.items():
                    for stream in streams:
                        super(ModuleIndex, self).add_known_stream(
                            module, stream
                        )

        ModuleIndex = override(ModuleIndex)
        __all__.append(ModuleIndex)

    if hasattr(Modulemd, "PackagerV3"):

        class PackagerV3(Modulemd.PackagerV3):
            def set_xmd(self, xmd):
                super(PackagerV3, self).set_xmd(
                    ModulemdUtil.python_to_variant(xmd)
                )

            def get_xmd(self):
                variant_xmd = super(PackagerV3, self).get_xmd()
                if variant_xmd is None:
                    return {}
                return variant_xmd.unpack()

        PackagerV3 = override(PackagerV3)
        __all__.append(PackagerV3)

    if hasattr(Modulemd, "ModuleStreamV2"):

        class ModuleStreamV2(Modulemd.ModuleStreamV2):
            def set_xmd(self, xmd):
                super(ModuleStreamV2, self).set_xmd(
                    ModulemdUtil.python_to_variant(xmd)
                )

            def get_xmd(self):
                variant_xmd = super(ModuleStreamV2, self).get_xmd()
                if variant_xmd is None:
                    return {}
                return variant_xmd.unpack()

        ModuleStreamV2 = override(ModuleStreamV2)
        __all__.append(ModuleStreamV2)

    if hasattr(Modulemd, "ModuleStreamV1"):

        class ModuleStreamV1(Modulemd.ModuleStreamV1):
            def set_xmd(self, xmd):
                super(ModuleStreamV1, self).set_xmd(
                    ModulemdUtil.python_to_variant(xmd)
                )

            def get_xmd(self):
                variant_xmd = super(ModuleStreamV1, self).get_xmd()
                if variant_xmd is None:
                    return {}
                return variant_xmd.unpack()

        ModuleStreamV1 = override(ModuleStreamV1)
        __all__.append(ModuleStreamV1)

    if hasattr(Modulemd, "ServiceLevel"):

        class ServiceLevel(Modulemd.ServiceLevel):
            def set_eol(self, eol):
                if isinstance(eol, datetime.date):
                    return super(ServiceLevel, self).set_eol_ymd(
                        eol.year, eol.month, eol.day
                    )

                raise TypeError(
                    "Expected datetime.date, but got %s."
                    % (type(eol).__name__)
                )

            def get_eol(self):
                eol = super(ServiceLevel, self).get_eol()
                if eol is None:
                    return None

                return datetime.date(
                    eol.get_year(), eol.get_month(), eol.get_day()
                )

        ServiceLevel = override(ServiceLevel)
        __all__.append(ServiceLevel)



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2017-2018 Stephen Gallagher
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

from os import path
import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
    from gi.repository.Modulemd import ModuleIndex
    from gi.repository import GLib
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


def debug_dump_failures(failures):
    if failures is None or len(failures) == 0:
        return
    print("{} YAML subdocuments were invalid".format(len(failures)))
    for f in failures:
        print(
            "Failed subdocument ({}):\n{}\n".format(
                str(f.get_gerror()), f.get_yaml()
            )
        )


class TestModuleIndex(TestBase):
    def test_constructors(self):
        # Test that the new() function works
        idx = ModuleIndex.new()
        self.assertIsNotNone(idx)
        self.assertListEqual(idx.get_module_names(), [])
        self.assertIsNone(idx.get_module("foo"))

    def test_read(self):
        idx = ModuleIndex.new()

        with open(
            path.join(self.source_root, "yaml_specs/modulemd_stream_v1.yaml"),
            "r",
        ) as v1:
            res, failures = idx.update_from_string(v1.read(), True)
            debug_dump_failures(failures)
            self.assertListEqual(failures, [])
            self.assertTrue(res)

        for fname in [
            "yaml_specs/modulemd_stream_v2.yaml",
            "yaml_specs/modulemd_translations_v1.yaml",
            "yaml_specs/modulemd_defaults_v1.yaml",
        ]:
            res, failures = idx.update_from_file(
                path.join(self.source_root, fname), True
            )
            debug_dump_failures(failures)
            self.assertListEqual(failures, [])
            self.assertTrue(res)

        res, failures = idx.update_from_file(
            path.join(self.test_data_path, "te.yaml"), True
        )
        self.assertFalse(res)
        self.assertEqual(len(failures), 1)
        self.assertIn(
            "No document type specified", str(failures[0].get_gerror())
        )
        self.assertMultiLineEqual(
            failures[0].get_yaml(),
            """---
summary: An example module
description: An example module.
profiles:
  profile_a: An example profile
...
""",
        )

        self.assertListEqual(idx.get_module_names(), ["foo"])

        mod_foo = idx.get_module("foo")
        self.assertTrue(mod_foo.validate())
        self.assertEqual(mod_foo.get_module_name(), "foo")
        with self.assertRaisesRegex(
            gi.repository.GLib.GError, "No streams matched"
        ):
            mod_foo.get_stream_by_NSVCA("a", 5, "c")
        self.assertEqual(len(mod_foo.get_all_streams()), 2)
        self.assertIsNotNone(mod_foo.get_defaults())

        defaults = mod_foo.get_defaults()
        self.assertIsNotNone(defaults)
        self.assertEqual(defaults.get_default_stream(), "x.y")

        stream = mod_foo.get_stream_by_NSVCA(
            "latest", 20160927144203, "c0ffee43"
        )
        self.assertIsNotNone(stream)
        self.assertEqual(
            stream.get_nsvc(), "foo:latest:20160927144203:c0ffee43"
        )
        self.assertEqual(
            stream.get_description(None),
            "A module for the demonstration of the metadata format. Also, the "
            "obligatory lorem ipsum dolor sit amet goes right here.",
        )
        self.assertEqual(
            stream.get_description("C"),
            "A module for the demonstration of the metadata format. Also, the "
            "obligatory lorem ipsum dolor sit amet goes right here.",
        )
        self.assertEqual(stream.get_description("en_GB"), "An example module.")

    def test_get_default_streams(self):
        idx = Modulemd.ModuleIndex.new()
        idx.update_from_file(path.join(self.test_data_path, "f29.yaml"), True)

        default_streams = idx.get_default_streams()
        self.assertIsNotNone(default_streams)

        self.assertIn("dwm", default_streams.keys())
        self.assertEqual("6.1", default_streams["dwm"])

        self.assertIn("stratis", default_streams.keys())
        self.assertEqual("1", default_streams["stratis"])

        self.assertNotIn("nodejs", default_streams.keys())

    def test_dump_empty_index(self):
        idx = Modulemd.ModuleIndex.new()

        with self.assertRaisesRegex(
            gi.repository.GLib.GError, "Index contains no modules."
        ):
            yaml = idx.dump_to_string()
            self.assertIsNone(yaml)

    def test_update_from_defaults_directory(self):
        idx = Modulemd.ModuleIndex.new()
        self.assertIsNotNone(idx)

        # First, verify that it works without overrides
        ret = idx.update_from_defaults_directory(
            path=path.join(self.test_data_path, "defaults"), strict=True
        )
        self.assertTrue(ret)

        # There should be three modules here: meson, ninja and nodejs
        self.assertEqual(len(idx.get_module_names()), 3)
        self.assertIn("meson", idx.get_module_names())
        self.assertIn("ninja", idx.get_module_names())
        self.assertIn("nodejs", idx.get_module_names())

        # Check default streams
        defs = idx.get_default_streams()
        self.assertIn("meson", defs)
        self.assertEqual("latest", defs["meson"])
        self.assertIn("ninja", defs)
        self.assertEqual("latest", defs["ninja"])
        self.assertNotIn("nodejs", defs)

        # Now add overrides too
        # First, verify that it works without overrides
        ret = idx.update_from_defaults_directory(
            path=path.join(self.test_data_path, "defaults"),
            overrides_path=path.join(
                self.test_data_path, "defaults", "overrides"
            ),
            strict=True,
        )
        self.assertTrue(ret)

        # There should be four modules here: meson, ninja, nodejs and
        # testmodule
        self.assertEqual(len(idx.get_module_names()), 4)
        self.assertIn("meson", idx.get_module_names())
        self.assertIn("ninja", idx.get_module_names())
        self.assertIn("nodejs", idx.get_module_names())
        self.assertIn("testmodule", idx.get_module_names())

        # Check default streams
        defs = idx.get_default_streams()
        self.assertIn("meson", defs)
        self.assertEqual("latest", defs["meson"])
        self.assertIn("ninja", defs)
        self.assertEqual("latest", defs["ninja"])
        self.assertIn("nodejs", defs)
        self.assertEqual("12", defs["nodejs"])
        self.assertIn("testmodule", defs)
        self.assertIn("teststream", defs["testmodule"])

        # Nonexistent defaults dir
        with self.assertRaisesRegex(GLib.Error, "No such file or directory"):
            ret = idx.update_from_defaults_directory(
                path=path.join(self.test_data_path, "defaults_nonexistent"),
                strict=True,
            )
            self.assertFalse(ret)

        # Nonexistent override dir
        with self.assertRaisesRegex(GLib.Error, "No such file or directory"):
            ret = idx.update_from_defaults_directory(
                path=path.join(self.test_data_path, "defaults"),
                overrides_path="overrides_nonexistent",
                strict=True,
            )
            self.assertFalse(ret)

    def test_clear_xmds(self):
        if "_overrides_module" in dir(Modulemd) and hasattr(
            gi.overrides.Modulemd, "ModuleStreamV2"
        ):
            idx = Modulemd.ModuleIndex.new()
            self.assertIsNotNone(idx)

            ret, failures = idx.update_from_string(
                """
---
document: modulemd
version: 2
data:
    name: foo
    stream: latest
    version: 1
    static_context: true
    context: c0ffee43
    arch: s390x
    summary: An example module
    description: A longer description
    license:
        module: MIT
    xmd:
        a_key: a_value
        another_key: another_value
        an_array:
          - a
          - b
...
---
document: modulemd
version: 2
data:
    name: foo
    stream: latest
    version: 2
    static_context: true
    context: c0ffee43
    arch: s390x
    summary: An example module
    description: A longer description
    license:
        module: MIT
    xmd:
        a_key: a_value
        another_key: another_value
        an_array:
          - a
          - b
...
""",
                strict=True,
            )
            self.assertEqual(len(failures), 0)

            module_names = idx.get_module_names()
            self.assertEqual(len(module_names), 1)
            self.assertEqual(module_names[0], "foo")

            module = idx.get_module(module_names[0])
            self.assertEqual(len(module.get_all_streams()), 2)

            ref_xmd = {
                "a_key": "a_value",
                "another_key": "another_value",
                "an_array": ["a", "b"],
            }

            for stream in module.get_all_streams():
                # Verify that the XMD data is present.
                xmd = stream.get_xmd()
                self.assertEqual(xmd, ref_xmd)

            idx.clear_xmds()

            for stream in module.get_all_streams():
                # Verify that the XMD data is gone now.
                self.assertEqual(stream.get_xmd(), {})


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2018 Red Hat, Inc.
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


class TestDependencies(TestBase):
    def test_constructor(self):
        # Test that the new() function works
        d = Modulemd.Dependencies.new()
        assert d
        assert d.get_buildtime_modules() == []
        with self.expect_signal(only_on_fatal_warnings=True):
            d.get_buildtime_streams("foobar123")
        assert d.get_runtime_modules() == []
        with self.expect_signal(only_on_fatal_warnings=True):
            d.get_runtime_streams("foobar123")

        # Test that keyword name is accepted
        d = Modulemd.Dependencies()
        assert d
        assert d.get_buildtime_modules() == []
        with self.expect_signal(only_on_fatal_warnings=True):
            d.get_buildtime_streams("foobar123")
        assert d.get_runtime_modules() == []
        with self.expect_signal(only_on_fatal_warnings=True):
            d.get_runtime_streams("foobar123")

    def test_copy(self):
        d_orig = Modulemd.Dependencies()
        d = d_orig.copy()
        assert d
        assert d.get_buildtime_modules() == []
        with self.expect_signal(only_on_fatal_warnings=True):
            d.get_buildtime_streams("foobar123")
        assert d.get_runtime_modules() == []
        with self.expect_signal(only_on_fatal_warnings=True):
            d.get_runtime_streams("foobar123")

        d_orig.add_buildtime_stream("buildmod1", "stream2")
        d_orig.add_buildtime_stream("buildmod1", "stream1")
        d_orig.set_empty_buildtime_dependencies_for_module("builddef")
        d_orig.add_runtime_stream("runmod1", "stream3")
        d_orig.add_runtime_stream("runmod1", "stream4")
        d_orig.set_empty_runtime_dependencies_for_module("rundef")

        d = d_orig.copy()
        assert d
        assert d.get_buildtime_modules() == ["builddef", "buildmod1"]
        assert d.get_buildtime_streams("builddef") == []
        assert d.get_buildtime_streams("buildmod1") == ["stream1", "stream2"]
        assert d.get_runtime_modules() == ["rundef", "runmod1"]
        assert d.get_runtime_streams("rundef") == []
        assert d.get_runtime_streams("runmod1") == ["stream3", "stream4"]


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2018 Red Hat, Inc.
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


class TestBuildopts(TestBase):
    def test_constructor(self):
        # Test that the new() function works
        b = Modulemd.Buildopts.new()
        assert b
        assert b.props.rpm_macros is None
        assert b.get_rpm_macros() is None
        assert b.get_rpm_whitelist() == []
        assert b.get_arches() == []

        # Test that init works with rpm_macros
        b = Modulemd.Buildopts(rpm_macros="Test macros")
        assert b
        assert b.props.rpm_macros == "Test macros"
        assert b.get_rpm_macros() == "Test macros"
        assert b.get_rpm_whitelist() == []
        assert b.get_arches() == []

    def test_copy(self):
        b_orig = Modulemd.Buildopts()
        b = b_orig.copy()
        assert b
        assert b.props.rpm_macros is None
        assert b.get_rpm_macros() is None
        assert b.get_rpm_whitelist() == []
        assert b.get_arches() == []

        b.add_rpm_to_whitelist("test2")
        b.add_rpm_to_whitelist("test3")
        b.add_rpm_to_whitelist("test1")
        b.add_arch("x86_64")
        b.add_arch("ppc64le")

        b = b_orig.copy()
        assert b
        # make sure lists added to b above got clobbered by copy
        assert b.get_rpm_whitelist() == []
        assert b.get_arches() == []

        b_orig.set_rpm_macros("Test macros")
        b_orig.add_rpm_to_whitelist("test2")
        b_orig.add_rpm_to_whitelist("test3")
        b_orig.add_rpm_to_whitelist("test1")
        b_orig.add_arch("x86_64")
        b_orig.add_arch("ppc64le")

        b = b_orig.copy()
        assert b
        assert b.props.rpm_macros == "Test macros"
        assert b.get_rpm_macros() == "Test macros"
        assert b.get_rpm_whitelist() == ["test1", "test2", "test3"]
        assert b.get_arches() == ["ppc64le", "x86_64"]

    def test_get_set_rpm_macros(self):
        b = Modulemd.Buildopts()

        assert b.props.rpm_macros is None
        assert b.get_rpm_macros() is None

        b.set_rpm_macros("foobar")
        assert b.props.rpm_macros == "foobar"
        assert b.get_rpm_macros() == "foobar"

        b.props.rpm_macros = "barfoo"
        assert b.props.rpm_macros == "barfoo"
        assert b.get_rpm_macros() == "barfoo"

        b.props.rpm_macros = None
        assert b.props.rpm_macros is None
        assert b.get_rpm_macros() is None

    def test_whitelist(self):
        b = Modulemd.Buildopts()

        assert b.get_rpm_whitelist() == []

        b.add_rpm_to_whitelist("test2")
        assert b.get_rpm_whitelist() == ["test2"]

        b.add_rpm_to_whitelist("test3")
        b.add_rpm_to_whitelist("test1")
        assert b.get_rpm_whitelist() == ["test1", "test2", "test3"]

        b.add_rpm_to_whitelist("test2")
        assert b.get_rpm_whitelist() == ["test1", "test2", "test3"]

        b.remove_rpm_from_whitelist("test1")
        assert b.get_rpm_whitelist() == ["test2", "test3"]

        b.clear_rpm_whitelist()
        assert b.get_rpm_whitelist() == []

    def test_arches(self):
        b = Modulemd.Buildopts()

        assert b.get_arches() == []

        b.add_arch("s390x")
        assert b.get_arches() == ["s390x"]

        b.add_arch("x86_64")
        b.add_arch("ppc64le")
        assert b.get_arches() == ["ppc64le", "s390x", "x86_64"]

        b.add_arch("s390x")
        assert b.get_arches() == ["ppc64le", "s390x", "x86_64"]

        b.remove_arch("ppc64le")
        assert b.get_arches() == ["s390x", "x86_64"]

        b.clear_arches()
        assert b.get_arches() == []


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2017-2018 Stephen Gallagher
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

from os import path
import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
    from gi.repository.Modulemd import ModuleIndex
    from gi.repository import GLib
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


class TestModule(TestBase):
    def test_search_streams(self):
        idx = Modulemd.ModuleIndex.new()
        idx.update_from_file(path.join(self.test_data_path, "f29.yaml"), True)
        module = idx.get_module("nodejs")

        self.assertEqual(len(module.search_streams("8", 0)), 1)
        self.assertEqual(len(module.search_streams("10", 0)), 1)

    def test_copy_with_obsoletes(self):
        idx = Modulemd.ModuleIndex.new()
        e = Modulemd.Obsoletes.new(1, 2, "testmodule", "teststream", "testmsg")
        e.set_obsoleted_by("module_obsoleter", "stream_obsoleter")
        idx.add_obsoletes(e)

        m = idx.get_module("testmodule")
        assert m
        assert m.get_module_name() == "testmodule"
        obsoletes_from_orig = m.get_newest_active_obsoletes("teststream", None)
        assert (
            obsoletes_from_orig.get_obsoleted_by_module_name()
            == "module_obsoleter"
        )

        m_copy = m.copy()
        assert m_copy.get_module_name() == "testmodule"
        obsoletes_from_copy = m_copy.get_newest_active_obsoletes(
            "teststream", None
        )
        assert (
            obsoletes_from_copy.get_obsoleted_by_module_name()
            == "module_obsoleter"
        )

    def test_adding_obsoletes_is_order_independent(self):
        obsoletes_without_context = """
---
document: modulemd-obsoletes
version: 1
data:
  module: nodejs
  stream: 10
  context: deadbeef
  modified: 2019-07-27T00:00Z
  message: test message
  obsoleted_by:
    module: nodejs
    stream: 12
...
"""
        obsoletes_with_context = """
---
document: modulemd-obsoletes
version: 1
data:
  module: nodejs
  stream: 10
  modified: 2019-09-27T00:00Z
  message: test message
  obsoleted_by:
    module: nodejs
    stream: 14
...
"""

        for ordered_yaml in [
            obsoletes_without_context + obsoletes_with_context,
            obsoletes_with_context + obsoletes_without_context,
        ]:
            idx = ModuleIndex.new()
            stream = Modulemd.ModuleStream.new(2, "nodejs", "10")
            stream.props.context = "deadbeef"
            res = idx.add_module_stream(stream)
            res, failures = idx.update_from_string(ordered_yaml, True)
            m = idx.get_module("nodejs")
            streams = m.get_all_streams()
            s = streams[0]
            assert (
                s.get_obsoletes_resolved().get_obsoleted_by_module_stream()
                == "14"
            )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2018 Red Hat, Inc.
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository.Modulemd import ComponentRpm
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


class TestComponentRpm(TestBase):
    def test_constructors(self):
        # Test that the new() function works
        rpm = ComponentRpm.new("testrpm")
        assert rpm
        assert rpm.props.buildorder == 0
        assert rpm.get_buildorder() == 0
        assert rpm.props.buildonly is False
        assert rpm.get_buildonly() is False
        assert rpm.props.name == "testrpm"
        assert rpm.get_name() == "testrpm"
        assert rpm.props.rationale is None
        assert rpm.get_rationale() is None
        assert rpm.props.ref is None
        assert rpm.get_ref() is None
        assert rpm.props.repository is None
        assert rpm.get_repository() is None
        assert rpm.props.cache is None
        assert rpm.get_cache() is None
        assert rpm.get_arches() == []
        assert rpm.get_multilib_arches() == []

        # Test that object instantiation works
        rpm = ComponentRpm(
            buildorder=42,
            buildonly=True,
            name="testrpm",
            rationale="Testing all the things",
            ref="someref",
            repository="somerepo",
            cache="somecache",
        )
        assert rpm
        assert rpm.props.buildorder == 42
        assert rpm.get_buildorder() == 42
        assert rpm.props.buildonly is True
        assert rpm.get_buildonly() is True
        assert rpm.props.name == "testrpm"
        assert rpm.get_name() == "testrpm"
        assert rpm.props.rationale == "Testing all the things"
        assert rpm.get_rationale() == "Testing all the things"
        assert rpm.props.ref == "someref"
        assert rpm.get_ref() == "someref"
        assert rpm.props.repository == "somerepo"
        assert rpm.get_repository() == "somerepo"
        assert rpm.props.cache == "somecache"
        assert rpm.get_cache() == "somecache"
        assert rpm.get_arches() == []
        assert rpm.get_multilib_arches() == []

    def test_copy(self):
        rpm_orig = ComponentRpm(
            buildorder=42,
            buildonly=True,
            name="testrpm",
            rationale="Testing all the things",
            ref="someref",
            repository="somerepo",
            cache="somecache",
        )
        rpm_orig.add_restricted_arch("x86_64")
        rpm_orig.add_restricted_arch("i386")
        rpm_orig.add_multilib_arch("ppc64le")
        rpm_orig.add_multilib_arch("s390x")

        rpm = rpm_orig.copy()
        assert rpm
        assert rpm.props.buildorder == 42
        assert rpm.get_buildorder() == 42
        assert rpm.props.buildonly is True
        assert rpm.get_buildonly() is True
        assert rpm.props.name == "testrpm"
        assert rpm.get_name() == "testrpm"
        assert rpm.props.rationale == "Testing all the things"
        assert rpm.get_rationale() == "Testing all the things"
        assert rpm.props.ref == "someref"
        assert rpm.get_ref() == "someref"
        assert rpm.props.repository == "somerepo"
        assert rpm.get_repository() == "somerepo"
        assert rpm.props.cache == "somecache"
        assert rpm.get_cache() == "somecache"
        self.assertListEqual(rpm.get_arches(), ["i386", "x86_64"])
        self.assertListEqual(rpm.get_multilib_arches(), ["ppc64le", "s390x"])

    def test_arches(self):
        rpm = ComponentRpm.new("testrpm")
        assert rpm

        rpm.add_restricted_arch("x86_64")
        rpm.add_restricted_arch("i386")
        self.assertListEqual(rpm.get_arches(), ["i386", "x86_64"])

        rpm.reset_arches()
        self.assertListEqual(rpm.get_arches(), [])

    def test_multilib(self):
        rpm = ComponentRpm.new("testrpm")
        assert rpm

        rpm.add_multilib_arch("ppc64le")
        rpm.add_multilib_arch("s390x")
        self.assertListEqual(rpm.get_multilib_arches(), ["ppc64le", "s390x"])

        rpm.reset_multilib_arches()
        self.assertListEqual(rpm.get_multilib_arches(), [])


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2018 Red Hat, Inc.
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


class TestProfile(TestBase):
    def test_constructor(self):
        # Test that the new() function works
        p = Modulemd.Profile.new("testprofile")
        assert p
        assert p.props.name == "testprofile"
        assert p.get_name() == "testprofile"
        assert p.get_description() is None
        assert p.get_rpms() == []

        # Test that keyword name is accepted
        p = Modulemd.Profile(name="testprofile")
        assert p
        assert p.props.name == "testprofile"
        assert p.get_name() == "testprofile"
        assert p.get_description() is None
        assert p.get_rpms() == []

        # Test that we fail without name
        with self.assertRaises(TypeError) as cm:
            Modulemd.Profile.new(None)
        assert "does not allow None as a value" in cm.exception.__str__()

        with self.expect_signal():
            Modulemd.Profile()

        with self.expect_signal():
            Modulemd.Profile(name=None)

    def test_copy(self):
        p_orig = Modulemd.Profile(name="testprofile")
        p = p_orig.copy()
        assert p
        assert p.props.name == "testprofile"
        assert p.get_name() == "testprofile"
        assert p.get_description() is None
        assert p.get_rpms() == []

        p_orig.set_description("Test profile")
        p.add_rpm("test2")
        p.add_rpm("test3")
        p.add_rpm("test1")

        p = p_orig.copy()
        assert p
        assert p.props.name == "testprofile"
        assert p.get_name() == "testprofile"
        assert p.get_description() == "Test profile"
        assert p.get_rpms() == ["test1", "test2", "test3"]

    def test_get_name(self):
        p = Modulemd.Profile(name="testprofile")

        assert p.get_name() == "testprofile"
        assert p.props.name == "testprofile"

        with self.expect_signal():
            p.props.name = "notadrill"

    def test_get_set_description(self):
        p = Modulemd.Profile(name="testprofile")

        assert p.get_description() is None

        p.set_description("foobar")
        assert p.get_description() == "foobar"

        p.set_description(None)
        assert p.get_description() is None

    def test_rpms(self):
        p = Modulemd.Profile(name="testprofile")

        assert p.get_rpms() == []

        p.add_rpm("test2")
        assert p.get_rpms() == ["test2"]

        p.add_rpm("test3")
        p.add_rpm("test1")
        assert p.get_rpms() == ["test1", "test2", "test3"]

        p.add_rpm("test2")
        assert p.get_rpms() == ["test1", "test2", "test3"]

        p.remove_rpm("test1")
        assert p.get_rpms() == ["test2", "test3"]


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2018 Red Hat, Inc.
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import os
import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


class TestDefaults(TestBase):
    def test_constructors(self):
        # Test that the new() function works
        defs = Modulemd.Defaults.new(Modulemd.DefaultsVersionEnum.ONE, "foo")
        assert defs

        assert defs.props.mdversion == Modulemd.DefaultsVersionEnum.ONE
        assert defs.get_mdversion() == Modulemd.DefaultsVersionEnum.ONE

        assert defs.props.module_name == "foo"
        assert defs.get_module_name() == "foo"

        # Test that we cannot instantiate directly
        with self.assertRaisesRegex(
            TypeError, "cannot create instance of abstract"
        ):
            Modulemd.Defaults()

        # Test with a zero mdversion
        with self.assertRaisesRegex(TypeError, "constructor returned NULL"):
            with self.expect_signal():
                defs = Modulemd.Defaults.new(0, "foo")

        # Test with an unknown mdversion
        with self.assertRaisesRegex(TypeError, "constructor returned NULL"):
            with self.expect_signal():
                defs = Modulemd.Defaults.new(
                    Modulemd.DefaultsVersionEnum.LATEST + 1, "foo"
                )

        # Test with no name
        with self.assertRaisesRegex(
            TypeError, "does not allow None as a value"
        ):
            defs = Modulemd.Defaults.new(
                Modulemd.DefaultsVersionEnum.ONE, None
            )

    def test_copy(self):
        defs = Modulemd.Defaults.new(Modulemd.DefaultsVersionEnum.ONE, "foo")
        assert defs

        copied_defs = defs.copy()
        assert copied_defs
        assert copied_defs.props.mdversion == defs.props.mdversion
        assert copied_defs.props.module_name == defs.props.module_name

    def test_mdversion(self):
        defs = Modulemd.Defaults.new(
            Modulemd.DefaultsVersionEnum.LATEST, "foo"
        )
        assert defs

        assert defs.props.mdversion == Modulemd.DefaultsVersionEnum.ONE
        assert defs.get_mdversion() == Modulemd.DefaultsVersionEnum.ONE

        # Ensure we cannot set the mdversion
        with self.assertRaisesRegex(TypeError, "is not writable"):
            defs.props.mdversion = 0

    def test_module_name(self):
        defs = Modulemd.Defaults.new(
            Modulemd.DefaultsVersionEnum.LATEST, "foo"
        )
        assert defs

        assert defs.props.module_name == "foo"
        assert defs.get_module_name() == "foo"

        # Ensure we cannot set the module_name
        with self.expect_signal():
            defs.props.module_name = None

    def test_modified(self):
        defs = Modulemd.Defaults.new(
            Modulemd.DefaultsVersionEnum.LATEST, "foo"
        )
        self.assertIsNotNone(defs)

        self.assertEqual(defs.get_modified(), 0)

        defs.set_modified(201901110830)

        self.assertEqual(defs.get_modified(), 201901110830)

        # Load a defaults object into an Index
        index = Modulemd.ModuleIndex.new()
        index.update_from_file(
            "%s/yaml_specs/modulemd_defaults_v1.yaml"
            % (os.getenv("MESON_SOURCE_ROOT")),
            True,
        )
        module_names = index.get_module_names()
        self.assertEqual(len(module_names), 1)

        defs = index.get_module(index.get_module_names()[0]).get_defaults()
        self.assertIsNotNone(defs)

        self.assertEqual(defs.get_modified(), 201812071200)

    def test_validate(self):
        defs = Modulemd.Defaults.new(
            Modulemd.DefaultsVersionEnum.LATEST, "foo"
        )
        assert defs

        assert defs.validate()

    def test_upgrade(self):
        defs = Modulemd.Defaults.new(Modulemd.DefaultsVersionEnum.ONE, "foo")
        assert defs

        # test upgrading to the same version
        upgraded_defs = defs.upgrade(Modulemd.DefaultsVersionEnum.ONE)
        assert upgraded_defs
        assert defs.props.mdversion == Modulemd.DefaultsVersionEnum.ONE
        assert defs.props.module_name == "foo"

        # test upgrading to the latest version
        upgraded_defs = defs.upgrade(Modulemd.DefaultsVersionEnum.LATEST)
        assert upgraded_defs
        assert defs.props.mdversion == Modulemd.DefaultsVersionEnum.LATEST
        assert defs.props.module_name == "foo"


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2018 Red Hat, Inc.
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

from contextlib import contextmanager
import signal
import os
import unittest


class TestBase(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestBase, self).__init__(*args, **kwargs)
        self._caught_signal = False

    @property
    def source_root(self):
        return os.getenv("MESON_SOURCE_ROOT")

    @property
    def test_data_path(self):
        return os.getenv("TEST_DATA_PATH")

    def _catch_signal(self, *sigargs):
        if self._caught_signal:
            raise AssertionError("Multiple signals were caught")
        self._caught_signal = True

    @contextmanager
    def expect_signal(
        self, expected_signal=signal.SIGTRAP, only_on_fatal_warnings=False
    ):
        expect_signal = (not only_on_fatal_warnings) or self.warnings_fatal

        self._caught_signal = False

        saved_signal = signal.signal(expected_signal, self._catch_signal)
        yield None
        signal.signal(expected_signal, saved_signal)
        if not self._caught_signal and expect_signal:
            raise AssertionError("No signal got caught")
        elif self._caught_signal and not expect_signal:
            raise AssertionError("Signal caught in non-warning state")

    @property
    def warnings_fatal(self):
        gdebug = os.getenv("G_DEBUG", "").split(",")
        return "fatal-warnings" in gdebug

    def assertRaisesRegex(self, *args, **kwargs):
        """Asserts that the message in a raised exception matches a regex.

        Args:
            The same as unittest.TestCase.assertRaisesRegex().
        """
        try:
            return super(TestBase, self).assertRaisesRegex(*args, **kwargs)
        except AttributeError:
            return self.assertRaisesRegexp(*args, **kwargs)



import json
import logging
import os
import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import GLib
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from copy import deepcopy
from base import TestBase

logging.basicConfig(level=logging.DEBUG)


class TestModulemdPackager(TestBase):
    def test_sanity(self):
        # Validate that the specification parses correctly
        stream = Modulemd.ModuleStream.read_file(
            os.path.join(
                self.source_root, "yaml_specs/modulemd_packager_v2.yaml"
            ),
            True,
        )
        self.assertIsNotNone(stream)

        # Once read in, a modulemd-packager document is a ModuleStream of the
        # same version.
        self.assertIs(type(stream), Modulemd.ModuleStreamV2)

    def test_read_minimal(self):
        minimal_valid = """---
document: modulemd-packager
version: 2
data:
  summary: A minimal valid module
  description: A minimalistic module description
  license:
    module:
      - DUMMY
...
"""
        stream = Modulemd.ModuleStream.read_string(minimal_valid, True)
        self.assertIsNotNone(stream)

        # Once read in, a modulemd-packager document is a ModuleStream of the
        # same version.
        self.assertIs(type(stream), Modulemd.ModuleStreamV2)

    def test_read_extra_values(self):
        minimal_valid = {
            "document": "modulemd-packager",
            "version": 2,
            "data": {
                "summary": "A minimal valid module",
                "description": "A minimalistic module description",
                "license": {"module": ["DUMMY1", "DUMMY2"]},
            },
        }

        extra_attrs = [
            "name",
            "stream",
            "context",
            "arch",
            "servicelevels",
            "xmd",
            "buildopts",
            "artifacts",
        ]

        for attr in extra_attrs:
            minimal_plus_extra = deepcopy(minimal_valid)

            # We can just pretend they're all scalars, because the parser will
            # halt before it tries to validate the value.
            minimal_plus_extra["data"][attr] = "invalid key"

            logging.debug(
                "YAML: {}".format(json.dumps(minimal_plus_extra, indent=2))
            )

            # This code takes advantage of the fact that JSON is a proper
            # subset of YAML, so we can avoid requiring pyYAML for the
            # tests.
            with self.assertRaises(gi.repository.GLib.GError) as cm:
                stream = Modulemd.ModuleStream.read_string(
                    json.dumps(minimal_plus_extra), True
                )

            gerror = cm.exception
            self.assertTrue(
                gerror.matches(
                    domain=Modulemd.yaml_error_quark(),
                    code=Modulemd.YamlError.UNKNOWN_ATTR,
                )
            )

        # Handle license.content as a special case since it's not directly
        # under `data`.
        minimal_plus_extra = deepcopy(minimal_valid)
        minimal_plus_extra["data"]["license"]["content"] = "invalid key"

        logging.debug(
            "YAML: {}".format(json.dumps(minimal_plus_extra, indent=2))
        )

        # This code takes advantage of the fact that JSON is a proper
        # subset of YAML, so we can avoid requiring pyYAML for the tests.
        with self.assertRaises(gi.repository.GLib.GError) as cm:
            stream = Modulemd.ModuleStream.read_string(
                json.dumps(minimal_plus_extra), True
            )

        gerror = cm.exception
        self.assertTrue(
            gerror.matches(
                domain=Modulemd.yaml_error_quark(),
                code=Modulemd.YamlError.UNKNOWN_ATTR,
            )
        )

    def test_fail_v1(self):
        minimal_v1 = {
            "document": "modulemd-packager",
            "version": 1,
            "data": {
                "summary": "A minimal valid module",
                "description": "A minimalistic module description",
                "license": {"module": ["DUMMY1", "DUMMY2"]},
            },
        }

        with self.assertRaises(gi.repository.GLib.Error) as cm:
            stream = Modulemd.ModuleStream.read_string(
                json.dumps(minimal_v1), True
            )
        gerror = cm.exception
        self.assertTrue(
            gerror.matches(
                domain=Modulemd.yaml_error_quark(),
                code=Modulemd.YamlError.INCONSISTENT,
            )
        )

    def test_v3_xmd(self):

        # We have a chicken-egg problem with overrides, since they can only
        # be tested if they are already installed. This means they need to
        # be run in the CI. In order to avoid changes to these tests or the
        # overrides breaking things, we'll skip them if the appropriate
        # override is not installed.
        if "_overrides_module" in dir(Modulemd) and hasattr(
            gi.overrides.Modulemd, "PackagerV3"
        ):

            # The XMD python tests can only be run against the installed lib
            # because the overrides that translate between python and GVariant
            # must be installed in /usr/lib/python*/site-packages/gi/overrides
            # or they are not included when importing Modulemd
            packager = Modulemd.PackagerV3.new()
            # An empty dictionary should be returned if no xmd value is set
            self.assertEqual(packager.get_xmd(), dict())

            xmd = {"outer_key": {"inner_key": ["scalar", "another_scalar"]}}

            packager.set_xmd(xmd)

            xmd_copy = packager.get_xmd()
            assert xmd_copy
            assert "outer_key" in xmd_copy
            assert "inner_key" in xmd_copy["outer_key"]
            assert "scalar" in xmd_copy["outer_key"]["inner_key"]
            assert "another_scalar" in xmd_copy["outer_key"]["inner_key"]

            # Verify that we can add content and save it back
            xmd["something"] = ["foo", "bar"]
            packager.set_xmd(xmd)

            xmd_copy = packager.get_xmd()
            assert xmd_copy
            assert "outer_key" in xmd_copy
            assert "inner_key" in xmd_copy["outer_key"]
            assert "scalar" in xmd_copy["outer_key"]["inner_key"]
            assert "another_scalar" in xmd_copy["outer_key"]["inner_key"]
            assert "something" in xmd_copy
            assert xmd_copy["something"] == ["foo", "bar"]


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2020 Red Hat, Inc.
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


class TestObsoletes(TestBase):
    def test_constructors(self):

        # Test that the new() function works
        o = Modulemd.Obsoletes.new(1, 2, "testmodule", "teststream", "testmsg")
        assert o
        assert o.props.mdversion == 1
        assert o.props.modified == 2
        assert o.props.module_name == "testmodule"
        assert o.props.module_stream == "teststream"
        assert o.props.message == "testmsg"

        # Test that keywords are accepted
        o = Modulemd.Obsoletes(
            mdversion=1,
            modified=42,
            module_name="testmodule2",
            module_stream="teststream2",
            message="testmessage2",
        )
        assert o
        assert o.validate()
        assert o.props.mdversion == 1
        assert o.props.modified == 42
        assert o.props.module_name == "testmodule2"
        assert o.props.module_stream == "teststream2"
        assert o.props.message == "testmessage2"

    def test_copy(self):
        o = Modulemd.Obsoletes.new(1, 2, "testmodule", "teststream", "testmsg")
        o_copy = o.copy()
        assert o_copy
        assert o_copy.validate()
        assert o_copy.props.mdversion == 1
        assert o_copy.props.modified == 2
        assert o_copy.props.module_name == "testmodule"
        assert o_copy.props.module_stream == "teststream"
        assert o_copy.props.message == "testmsg"


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# This file is part of libmodulemd
# Copyright (C) 2018-2020 Red Hat, Inc.
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import os
import sys

try:
    import unittest
    import gi

    if os.getenv("MMD_TEST_INSTALLED_LIBS") != "TRUE":
        gi._overridesdir = os.path.join(
            os.getenv("MESON_SOURCE_ROOT"),
            "bindings",
            "python",
            "gi",
            "overrides",
        )

    gi.require_version("Modulemd", "2.0")
    from gi.repository import GLib
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase

modulestream_versions = [
    Modulemd.ModuleStreamVersionEnum.ONE,
    Modulemd.ModuleStreamVersionEnum.TWO,
]


class TestModuleStream(TestBase):
    def test_constructors(self):
        for version in modulestream_versions:

            # Test that the new() function works
            stream = Modulemd.ModuleStream.new(version, "foo", "latest")
            assert stream
            assert isinstance(stream, Modulemd.ModuleStream)

            assert stream.props.mdversion == version
            assert stream.get_mdversion() == version
            assert stream.props.module_name == "foo"
            assert stream.get_module_name() == "foo"
            assert stream.props.stream_name == "latest"
            assert stream.get_stream_name() == "latest"

            # Test that the new() function works without a stream name
            stream = Modulemd.ModuleStream.new(version, "foo")
            assert stream
            assert isinstance(stream, Modulemd.ModuleStream)

            assert stream.props.mdversion == version
            assert stream.get_mdversion() == version
            assert stream.props.module_name == "foo"
            assert stream.get_module_name() == "foo"
            assert stream.props.stream_name is None
            assert stream.get_stream_name() is None

            # Test that the new() function works with no module name
            stream = Modulemd.ModuleStream.new(version, None, "latest")
            assert stream
            assert isinstance(stream, Modulemd.ModuleStream)

            assert stream.props.mdversion == version
            assert stream.get_mdversion() == version
            assert stream.props.module_name is None
            assert stream.get_module_name() is None
            assert stream.props.stream_name == "latest"
            assert stream.get_stream_name() == "latest"

            # Test that the new() function works with no module or stream
            stream = Modulemd.ModuleStream.new(version)
            assert stream
            assert isinstance(stream, Modulemd.ModuleStream)

            assert stream.props.mdversion == version
            assert stream.get_mdversion() == version
            assert stream.props.module_name is None
            assert stream.get_module_name() is None
            assert stream.props.stream_name is None
            assert stream.get_stream_name() is None

        # Test that we cannot instantiate directly
        with self.assertRaisesRegex(
            TypeError, "cannot create instance of abstract"
        ):
            Modulemd.ModuleStream()

        # Test with a zero mdversion
        with self.assertRaisesRegex(TypeError, "constructor returned NULL"):
            with self.expect_signal():
                defs = Modulemd.ModuleStream.new(0)

        # Test with an unknown mdversion
        with self.assertRaisesRegex(TypeError, "constructor returned NULL"):
            with self.expect_signal():
                defs = Modulemd.ModuleStream.new(
                    Modulemd.ModuleStreamVersionEnum.LATEST + 1
                )

    def test_copy(self):
        for version in modulestream_versions:

            # Test that copying a stream with a stream name works
            stream = Modulemd.ModuleStream.new(version, "foo", "stable")
            copied_stream = stream.copy()

            assert copied_stream.props.module_name == stream.props.module_name
            assert copied_stream.get_module_name() == stream.get_module_name()
            assert copied_stream.props.stream_name == stream.props.stream_name
            assert copied_stream.get_stream_name() == stream.get_stream_name()

            # Test that copying a stream without a stream name works
            stream = Modulemd.ModuleStream.new(version, "foo")
            copied_stream = stream.copy()

            assert copied_stream.props.module_name == stream.props.module_name
            assert copied_stream.get_module_name() == stream.get_module_name()
            assert copied_stream.props.stream_name == stream.props.stream_name
            assert copied_stream.get_stream_name() == stream.get_stream_name()

            # Test that copying a stream and changing the stream works
            stream = Modulemd.ModuleStream.new(version, "foo", "stable")
            copied_stream = stream.copy(module_stream="latest")

            assert copied_stream.props.module_name == stream.props.module_name
            assert copied_stream.get_module_name() == stream.get_module_name()
            assert copied_stream.props.stream_name != stream.props.stream_name
            assert copied_stream.get_stream_name() != stream.get_stream_name()
            assert copied_stream.props.stream_name == "latest"
            assert copied_stream.get_stream_name() == "latest"

            # Test that copying a stream without a module name works
            stream = Modulemd.ModuleStream.new(version, None, "stable")
            copied_stream = stream.copy()

            assert copied_stream.props.module_name == stream.props.module_name
            assert copied_stream.get_module_name() == stream.get_module_name()
            assert copied_stream.props.stream_name == stream.props.stream_name
            assert copied_stream.get_stream_name() == stream.get_stream_name()

            # Test that copying a stream and changing the name works
            stream = Modulemd.ModuleStream.new(version, "foo", "stable")
            copied_stream = stream.copy(module_name="bar")

            assert copied_stream.props.module_name == "bar"
            assert copied_stream.get_module_name() == "bar"
            assert copied_stream.props.stream_name == stream.props.stream_name
            assert copied_stream.get_stream_name() == stream.get_stream_name()

            # Test the version and context
            assert copied_stream.props.version == 0
            assert copied_stream.get_version() == 0
            assert copied_stream.props.context is None
            assert copied_stream.get_context() is None

            # Set a version and check the copy
            stream.props.version = 42
            copied_stream = stream.copy()
            assert copied_stream.props.version == 42
            assert copied_stream.get_version() == 42

            # Set a context and check the copy
            stream.props.context = "c0ffee42"
            copied_stream = stream.copy()
            assert copied_stream.props.context == "c0ffee42"
            assert copied_stream.get_context() == "c0ffee42"

    def test_nsvc(self):
        for version in modulestream_versions:
            # First test that NSVC is None for a module with no name
            stream = Modulemd.ModuleStream.new(version)
            assert stream.get_nsvc() is None

            # Next, test for no stream name
            stream = Modulemd.ModuleStream.new(version, "modulename")
            assert stream.get_nsvc() is None

            # Now with valid module and stream names
            stream = Modulemd.ModuleStream.new(
                version, "modulename", "streamname"
            )
            assert stream.get_nsvc() == "modulename:streamname:0"

            # Add a version number
            stream.props.version = 42
            assert stream.get_nsvc() == "modulename:streamname:42"

            # Add a context
            stream.props.context = "deadbeef"
            assert stream.get_nsvc() == "modulename:streamname:42:deadbeef"

    def test_nsvca(self):
        for version in modulestream_versions:
            # First test that NSVCA is None for a module with no name
            stream = Modulemd.ModuleStream.new(version)
            self.assertIsNone(stream.get_NSVCA())

            # Next, test for no stream name
            stream = Modulemd.ModuleStream.new(version, "modulename")
            self.assertEqual(stream.get_NSVCA(), "modulename")

            # Now with valid module and stream names
            stream = Modulemd.ModuleStream.new(
                version, "modulename", "streamname"
            )
            self.assertEqual(stream.get_NSVCA(), "modulename:streamname")

            # Add a version number
            stream.props.version = 42
            self.assertEqual(stream.get_NSVCA(), "modulename:streamname:42")

            # Add a context
            stream.props.context = "deadbeef"
            self.assertEqual(
                stream.get_NSVCA(), "modulename:streamname:42:deadbeef"
            )

            # Add an architecture
            stream.props.arch = "x86_64"
            self.assertEqual(
                stream.get_NSVCA(), "modulename:streamname:42:deadbeef:x86_64"
            )

            # Now try removing some of the bits in the middle
            stream.props.context = None
            self.assertEqual(
                stream.get_NSVCA(), "modulename:streamname:42::x86_64"
            )

            stream = Modulemd.ModuleStream.new(version, "modulename")
            stream.props.arch = "x86_64"
            self.assertEqual(stream.get_NSVCA(), "modulename::::x86_64")

            stream.props.version = 2019
            self.assertEqual(stream.get_NSVCA(), "modulename::2019::x86_64")
            # Add a context
            stream.props.context = "feedfeed"
            self.assertEqual(
                stream.get_NSVCA(), "modulename::2019:feedfeed:x86_64"
            )

    def test_arch(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)

            # Check the defaults
            assert stream.props.arch is None
            assert stream.get_arch() is None

            # Test property setting
            stream.props.arch = "x86_64"
            assert stream.props.arch == "x86_64"
            assert stream.get_arch() == "x86_64"

            # Test set_arch()
            stream.set_arch("ppc64le")
            assert stream.props.arch == "ppc64le"
            assert stream.get_arch() == "ppc64le"

            # Test setting it to None
            stream.props.arch = None
            assert stream.props.arch is None
            assert stream.get_arch() is None

    def test_buildopts(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)

            # Check the defaults
            assert stream.props.buildopts is None
            assert stream.get_buildopts() is None

            buildopts = Modulemd.Buildopts()
            buildopts.props.rpm_macros = "%demomacro 1"
            stream.set_buildopts(buildopts)

            assert stream.props.buildopts is not None
            assert stream.props.buildopts is not None
            assert stream.props.buildopts.props.rpm_macros == "%demomacro 1"

    def test_community(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)

            # Check the defaults
            assert stream.props.community is None
            assert stream.get_community() is None

            # Test property setting
            stream.props.community = "http://example.com"
            assert stream.props.community == "http://example.com"
            assert stream.get_community() == "http://example.com"

            # Test set_community()
            stream.set_community("http://redhat.com")
            assert stream.props.community == "http://redhat.com"
            assert stream.get_community() == "http://redhat.com"

            # Test setting it to None
            stream.props.community = None
            assert stream.props.community is None
            assert stream.get_community() is None

    def test_description(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)

            # Check the defaults
            assert stream.get_description(locale="C") is None

            # Test set_description()
            stream.set_description("A different description")
            assert (
                stream.get_description(locale="C") == "A different description"
            )

            # Test setting it to None
            stream.set_description(None)
            assert stream.get_description(locale="C") is None

    def test_documentation(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)

            # Check the defaults
            assert stream.props.documentation is None
            assert stream.get_documentation() is None

            # Test property setting
            stream.props.documentation = "http://example.com"
            assert stream.props.documentation == "http://example.com"
            assert stream.get_documentation() == "http://example.com"

            # Test set_documentation()
            stream.set_documentation("http://redhat.com")
            assert stream.props.documentation == "http://redhat.com"
            assert stream.get_documentation() == "http://redhat.com"

            # Test setting it to None
            stream.props.documentation = None
            assert stream.props.documentation is None
            assert stream.get_documentation() is None

            # Test unicode characters
            unicode_test_str = (
                "À϶￥🌭∮⇒⇔¬β∀₂⌀ıəˈ⍳⍴V)═€ίζησθლბშიнстемองจึองታሽ።ደለᚢᛞᚦᚹ⠳⠞⠊⠎▉▒▒▓😃"
            )

            stream.props.documentation = unicode_test_str
            assert stream.props.documentation == unicode_test_str
            assert stream.get_documentation() == unicode_test_str

    def test_summary(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)

            # Check the defaults
            assert stream.get_summary(locale="C") is None

            # Test set_summary()
            stream.set_summary("A different summary")
            assert stream.get_summary(locale="C") == "A different summary"

            # Test setting it to None
            stream.set_summary(None)
            assert stream.get_summary(locale="C") is None

    def test_tracker(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)

            # Check the defaults
            assert stream.props.tracker is None
            assert stream.get_tracker() is None

            # Test property setting
            stream.props.tracker = "http://example.com"
            assert stream.props.tracker == "http://example.com"
            assert stream.get_tracker() == "http://example.com"

            # Test set_tracker()
            stream.set_tracker("http://redhat.com")
            assert stream.props.tracker == "http://redhat.com"
            assert stream.get_tracker() == "http://redhat.com"

            # Test setting it to None
            stream.props.tracker = None
            assert stream.props.tracker is None
            assert stream.get_tracker() is None

    def test_components(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)

            # Add an RPM component to a stream
            rpm_comp = Modulemd.ComponentRpm(name="rpmcomponent")
            stream.add_component(rpm_comp)
            assert "rpmcomponent" in stream.get_rpm_component_names()
            retrieved_comp = stream.get_rpm_component("rpmcomponent")
            assert retrieved_comp
            assert retrieved_comp.props.name == "rpmcomponent"

            # Add a Module component to a stream
            mod_comp = Modulemd.ComponentModule(name="modulecomponent")
            stream.add_component(mod_comp)
            assert "modulecomponent" in stream.get_module_component_names()
            retrieved_comp = stream.get_module_component("modulecomponent")
            assert retrieved_comp
            assert retrieved_comp.props.name == "modulecomponent"

            # Remove an RPM component from a stream
            stream.remove_rpm_component("rpmcomponent")

            # Remove a Module component from a stream
            stream.remove_module_component("modulecomponent")

    def test_licenses(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)

            stream.add_content_license("DUMMY1")
            assert "DUMMY1" in stream.get_content_licenses()

            stream.add_module_license("DUMMY2")
            assert "DUMMY2" in stream.get_module_licenses()

            stream.remove_content_license("DUMMY1")
            stream.remove_module_license("DUMMY2")

    def test_profiles(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version, "sssd")

            profile = Modulemd.Profile(name="client")
            profile.add_rpm("sssd-client")

            stream.add_profile(profile)
            assert len(stream.get_profile_names()) == 1
            assert "client" in stream.get_profile_names()
            assert "sssd-client" in stream.get_profile("client").get_rpms()

            stream.clear_profiles()
            assert len(stream.get_profile_names()) == 0

    def test_rpm_api(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version, "sssd")

            stream.add_rpm_api("sssd-common")
            assert "sssd-common" in stream.get_rpm_api()

            stream.remove_rpm_api("sssd-common")
            assert len(stream.get_rpm_api()) == 0

    def test_rpm_artifacts(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)

            stream.add_rpm_artifact("bar-0:1.23-1.module_deadbeef.x86_64")
            assert (
                "bar-0:1.23-1.module_deadbeef.x86_64"
                in stream.get_rpm_artifacts()
            )

            stream.remove_rpm_artifact("bar-0:1.23-1.module_deadbeef.x86_64")
            assert len(stream.get_rpm_artifacts()) == 0

    def test_rpm_filters(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)

            stream.add_rpm_filter("foo")
            stream.add_rpm_filter("bar")
            assert "foo" in stream.get_rpm_filters()
            assert "bar" in stream.get_rpm_filters()
            assert len(stream.get_rpm_filters()) == 2

            stream.remove_rpm_filter("bar")
            assert "foo" in stream.get_rpm_filters()
            assert len(stream.get_rpm_filters()) == 1

            stream.clear_rpm_filters()
            assert len(stream.get_rpm_filters()) == 0

    def test_demodularized_rpms(self):
        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.new(version)
            # demodularized_rpms are supported since v2
            if version < Modulemd.ModuleStreamVersionEnum.TWO:
                continue
            stream.add_demodularized_rpm("foo")
            stream.add_demodularized_rpm("bar")
            assert "foo" in stream.get_demodularized_rpms()
            assert "bar" in stream.get_demodularized_rpms()
            assert len(stream.get_demodularized_rpms()) == 2

            stream.remove_demodularized_rpm("bar")
            assert "foo" in stream.get_demodularized_rpms()
            assert len(stream.get_demodularized_rpms()) == 1

            stream.clear_demodularized_rpms()
            assert len(stream.get_demodularized_rpms()) == 0

    def test_servicelevels(self):
        for version in modulestream_versions:
            # servicelevels are not supported after v2
            if version > Modulemd.ModuleStreamVersionEnum.TWO:
                continue
            stream = Modulemd.ModuleStream.new(version)
            sl = Modulemd.ServiceLevel.new("rawhide")
            sl.set_eol_ymd(1980, 3, 2)

            stream.add_servicelevel(sl)

            assert "rawhide" in stream.get_servicelevel_names()

            retrieved_sl = stream.get_servicelevel("rawhide")
            assert retrieved_sl.props.name == "rawhide"
            assert retrieved_sl.get_eol_as_string() == "1980-03-02"

    def test_v1_eol(self):
        stream = Modulemd.ModuleStreamV1.new()
        eol = GLib.Date.new_dmy(3, 2, 1998)

        stream.set_eol(eol)
        retrieved_eol = stream.get_eol()

        assert retrieved_eol.get_day() == 3
        assert retrieved_eol.get_month() == 2
        assert retrieved_eol.get_year() == 1998

        sl = stream.get_servicelevel("rawhide")
        assert sl.get_eol_as_string() == "1998-02-03"

    def test_v1_dependencies(self):
        stream = Modulemd.ModuleStreamV1.new()
        stream.add_buildtime_requirement("testmodule", "stable")

        assert len(stream.get_buildtime_modules()) == 1
        assert "testmodule" in stream.get_buildtime_modules()

        assert (
            stream.get_buildtime_requirement_stream("testmodule") == "stable"
        )

        stream.add_runtime_requirement("testmodule", "latest")
        assert len(stream.get_runtime_modules()) == 1
        assert "testmodule" in stream.get_runtime_modules()
        assert stream.get_runtime_requirement_stream("testmodule") == "latest"

    def test_v2_dependencies(self):
        stream = Modulemd.ModuleStreamV2.new()
        deps = Modulemd.Dependencies()

        deps.add_buildtime_stream("foo", "stable")
        deps.set_empty_runtime_dependencies_for_module("bar")
        stream.add_dependencies(deps)

        assert len(stream.get_dependencies()) == 1
        assert len(stream.get_dependencies()) == 1

        assert "foo" in stream.get_dependencies()[0].get_buildtime_modules()

        assert "stable" in stream.get_dependencies()[0].get_buildtime_streams(
            "foo"
        )

        assert "bar" in stream.get_dependencies()[0].get_runtime_modules()

        retrieved_deps = stream.get_dependencies()
        stream.clear_dependencies()
        self.assertEqual(len(retrieved_deps), 1)
        self.assertEqual(len(stream.get_dependencies()), 0)

        stream.add_dependencies(deps)
        self.assertEqual(len(stream.get_dependencies()), 1)

        stream.remove_dependencies(deps)
        self.assertEqual(len(stream.get_dependencies()), 0)

    def test_xmd(self):

        for version in modulestream_versions:
            # We have a chicken-egg problem with overrides, since they can only
            # be tested if they are already installed. This means they need to
            # be run in the CI. In order to avoid changes to these tests or the
            # overrides breaking things, we'll skip them if the appropriate
            # override is not installed.
            if "_overrides_module" in dir(Modulemd) and hasattr(
                gi.overrides.Modulemd,
                type(Modulemd.ModuleStream.new(version)).__name__,
            ):

                # The XMD python tests can only be run against the installed lib
                # because the overrides that translate between python and GVariant
                # must be installed in /usr/lib/python*/site-packages/gi/overrides
                # or they are not included when importing Modulemd
                stream = Modulemd.ModuleStream.new(version, "foo", "bar")
                # An empty dictionary should be returned if no xmd value is set
                self.assertEqual(stream.get_xmd(), dict())

                xmd = {
                    "outer_key": {"inner_key": ["scalar", "another_scalar"]}
                }

                stream.set_xmd(xmd)

                xmd_copy = stream.get_xmd()
                assert xmd_copy
                assert "outer_key" in xmd_copy
                assert "inner_key" in xmd_copy["outer_key"]
                assert "scalar" in xmd_copy["outer_key"]["inner_key"]
                assert "another_scalar" in xmd_copy["outer_key"]["inner_key"]

                # Verify that we can add content and save it back
                xmd["something"] = ["foo", "bar"]
                stream.set_xmd(xmd)

                stream.set_summary("foo")
                stream.set_description("bar")
                stream.add_module_license("DUMMY")
                if hasattr(stream, "set_platform"):
                    stream.set_platform("f33")

                index = Modulemd.ModuleIndex()
                index.add_module_stream(stream)

                out_yaml = index.dump_to_string()

                self.assertIsNotNone(out_yaml)

    def test_upgrade_v1_to_v2(self):
        v1_stream = Modulemd.ModuleStreamV1.new("SuperModule", "latest")
        v1_stream.set_context("ctx")
        v1_stream.set_summary("Summary")
        v1_stream.set_description("Description")
        v1_stream.add_module_license("DUMMY")

        v1_stream.add_buildtime_requirement("ModuleA", "streamZ")
        v1_stream.add_buildtime_requirement("ModuleB", "streamY")
        v1_stream.add_runtime_requirement("ModuleA", "streamZ")
        v1_stream.add_runtime_requirement("ModuleB", "streamY")
        v1_stream.add_runtime_requirement("platform", "f33")

        v2_stream = v1_stream.upgrade(Modulemd.ModuleStreamVersionEnum.TWO)
        self.assertIsNotNone(v2_stream)

        idx = Modulemd.ModuleIndex.new()
        idx.add_module_stream(v2_stream)

        self.assertEqual(
            idx.dump_to_string(),
            """---
document: modulemd
version: 2
data:
  name: SuperModule
  stream: \"latest\"
  context: ctx
  summary: Summary
  description: >-
    Description
  license:
    module:
    - DUMMY
  dependencies:
  - buildrequires:
      ModuleA: [streamZ]
      ModuleB: [streamY]
    requires:
      ModuleA: [streamZ]
      ModuleB: [streamY]
      platform: [f33]
...
""",
        )

    def test_v2_yaml(self):
        yaml = """
---
document: modulemd
version: 2
data:
  name: modulename
  stream: \"streamname\"
  version: 1
  context: c0ffe3
  arch: x86_64
  summary: Module Summary
  description: >-
    Module Description
  api:
    rpms:
      - rpm_a
      - rpm_b
  filter:
    rpms: rpm_c

  demodularized:
    rpms:
      - rpm_d

  artifacts:
    rpms:
      - bar-0:1.23-1.module_deadbeef.x86_64

  servicelevels:
    rawhide: {}
    production:
      eol: 2099-12-31

  license:
    content:
      - DUMMY1
      - DUMMY2
    module: DUMMY3

  dependencies:
    - buildrequires:
          platform: [-f27, -f28, -epel7]
      requires:
          platform: [-f27, -f28, -epel7]
    - buildrequires:
          platform: [f27]
          buildtools: [v1, v2]
          compatible: [v3]
      requires:
          platform: [f27]
          compatible: [v3, v4]
    - buildrequires:
          platform: [f28]
      requires:
          platform: [f28]
          runtime: [a, b]
    - buildrequires:
          platform: [epel7]
          extras: []
          moreextras: [foo, bar]
      requires:
          platform: [epel7]
          extras: []
          moreextras: [foo, bar]
  references:
        community: http://www.example.com/
        documentation: http://www.example.com/
        tracker: http://www.example.com/
  profiles:
        default:
            rpms:
                - bar
                - bar-extras
                - baz
        container:
            rpms:
                - bar
                - bar-devel
        minimal:
            description: Minimal profile installing only the bar package.
            rpms:
                - bar
        buildroot:
            rpms:
                - bar-devel
        srpm-buildroot:
            rpms:
                - bar-extras
  buildopts:
        rpms:
            macros: |
                %demomacro 1
                %demomacro2 %{demomacro}23
            whitelist:
                - fooscl-1-bar
                - fooscl-1-baz
                - xxx
                - xyz
        arches: [i686, x86_64]
  components:
        rpms:
            bar:
                rationale: We need this to demonstrate stuff.
                repository: https://pagure.io/bar.git
                cache: https://example.com/cache
                ref: 26ca0c0
            baz:
                rationale: This one is here to demonstrate other stuff.
            xxx:
                rationale: xxx demonstrates arches and multilib.
                arches: [i686, x86_64]
                multilib: [x86_64]
            xyz:
                rationale: xyz is a bundled dependency of xxx.
                buildorder: 10
        modules:
            includedmodule:
                rationale: Included in the stack, just because.
                repository: https://pagure.io/includedmodule.git
                ref: somecoolbranchname
                buildorder: 100
  xmd:
        some_key: some_data
        some_list:
            - a
            - b
        some_dict:
            a: alpha
            b: beta
            some_other_list:
                - c
                - d
            some_other_dict:
                another_key: more_data
                yet_another_key:
                    - this
                    - is
                    - getting
                    - silly
        can_bool: TRUE
...
"""
        stream = Modulemd.ModuleStream.read_string(yaml, True)

        assert stream is not None
        assert stream.props.module_name == "modulename"
        assert stream.props.stream_name == "streamname"
        assert stream.props.version == 1
        assert stream.props.context == "c0ffe3"
        assert stream.props.arch == "x86_64"
        assert stream.get_summary(locale="C") == "Module Summary"
        assert stream.get_description(locale="C") == "Module Description"

        assert "rpm_a" in stream.get_rpm_api()
        assert "rpm_b" in stream.get_rpm_api()

        assert "rpm_c" in stream.get_rpm_filters()

        assert "rpm_d" in stream.get_demodularized_rpms()

        assert (
            "bar-0:1.23-1.module_deadbeef.x86_64" in stream.get_rpm_artifacts()
        )

        assert "rawhide" in stream.get_servicelevel_names()
        assert "production" in stream.get_servicelevel_names()

        sl = stream.get_servicelevel("rawhide")
        assert sl is not None
        assert sl.props.name == "rawhide"
        assert sl.get_eol() is None

        sl = stream.get_servicelevel("production")
        assert sl is not None
        assert sl.props.name == "production"
        assert sl.get_eol() is not None
        assert sl.get_eol_as_string() == "2099-12-31"

        assert "DUMMY1" in stream.get_content_licenses()
        assert "DUMMY2" in stream.get_content_licenses()

        assert len(stream.get_dependencies()) == 4

        assert stream.props.community == "http://www.example.com/"
        assert stream.props.documentation == "http://www.example.com/"
        assert stream.props.tracker == "http://www.example.com/"

        assert len(stream.get_profile_names()) == 5

        buildopts = stream.get_buildopts()
        assert buildopts is not None

        assert (
            "%demomacro 1\n%demomacro2 %{demomacro}23\n"
            == buildopts.props.rpm_macros
        )
        assert "fooscl-1-bar" in buildopts.get_rpm_whitelist()
        assert "fooscl-1-baz" in buildopts.get_rpm_whitelist()
        assert "xxx" in buildopts.get_rpm_whitelist()
        assert "xyz" in buildopts.get_rpm_whitelist()
        assert "i686" in buildopts.get_arches()
        assert "x86_64" in buildopts.get_arches()

        if os.getenv("MMD_TEST_INSTALLED_LIB"):
            # The XMD python tests can only be run against the installed
            # lib because the overrides that translate between python and
            # GVariant must be installed in
            # /usr/lib/python*/site-packages/gi/overrides
            # or they are not included when importing Modulemd
            xmd = stream.get_xmd()
            assert xmd is not None

            assert "some_key" in xmd
            assert xmd["some_key"] == "some_data"

            assert "some_list" in xmd

            assert "a" in xmd["some_list"]
            assert "b" in xmd["some_list"]

            assert "some_dict" in xmd
            assert "a" in xmd["some_dict"]
            assert xmd["some_dict"]["a"] == "alpha"

            assert "some_other_dict" in xmd["some_dict"]
            assert "yet_another_key" in xmd["some_dict"]["some_other_dict"]
            assert (
                "silly"
                in xmd["some_dict"]["some_other_dict"]["yet_another_key"]
            )

            assert "can_bool" in xmd
            assert xmd["can_bool"] is True

        # Validate a trivial modulemd
        trivial_yaml = """
---
document: modulemd
version: 2
data:
    summary: Trivial Summary
    description: >-
      Trivial Description
    license:
        module: DUMMY
...
"""

        stream = Modulemd.ModuleStream.read_string(trivial_yaml, True)
        assert stream

        # Sanity check of spec.v2.yaml
        stream = Modulemd.ModuleStream.read_file(
            os.path.join(
                self.source_root, "yaml_specs/modulemd_stream_v2.yaml"
            ),
            True,
        )
        assert stream

    def test_v1_yaml(self):
        for version in modulestream_versions:
            yaml = """
---
document: modulemd
version: 1
data:
  name: modulename
  stream: streamname
  version: 1
  context: c0ffe3
  arch: x86_64
  summary: Module Summary
  description: >-
    Module Description
  api:
    rpms:
      - rpm_a
      - rpm_b
  filter:
    rpms: rpm_c

  artifacts:
    rpms:
      - bar-0:1.23-1.module_deadbeef.x86_64

  eol: 2033-08-04
  servicelevels:
    foo: {}
    production:
      eol: 2099-12-31

  license:
    content:
      - DUMMY1
      - DUMMY2
    module: DUMMY3

  dependencies:
        buildrequires:
            platform: and-its-stream-name
            extra-build-env: and-its-stream-name-too
        requires:
            runtimeplatform: and-its-stream-name-2

  references:
        community: http://www.example.com/
        documentation: http://www.example.com/
        tracker: http://www.example.com/
  profiles:
        default:
            rpms:
                - bar
                - bar-extras
                - baz
        container:
            rpms:
                - bar
                - bar-devel
        minimal:
            description: Minimal profile installing only the bar package.
            rpms:
                - bar
        buildroot:
            rpms:
                - bar-devel
        srpm-buildroot:
            rpms:
                - bar-extras
  buildopts:
        rpms:
            macros: |
                %demomacro 1
                %demomacro2 %{demomacro}23
            whitelist:
                - fooscl-1-bar
                - fooscl-1-baz
                - xxx
                - xyz
  components:
        rpms:
            bar:
                rationale: We need this to demonstrate stuff.
                repository: https://pagure.io/bar.git
                cache: https://example.com/cache
                ref: 26ca0c0
            baz:
                rationale: This one is here to demonstrate other stuff.
            xxx:
                rationale: xxx demonstrates arches and multilib.
                arches: [i686, x86_64]
                multilib: [x86_64]
            xyz:
                rationale: xyz is a bundled dependency of xxx.
                buildorder: 10
        modules:
            includedmodule:
                rationale: Included in the stack, just because.
                repository: https://pagure.io/includedmodule.git
                ref: somecoolbranchname
                buildorder: 100
  xmd:
        some_key: some_data
        some_list:
            - a
            - b
        some_dict:
            a: alpha
            b: beta
            some_other_list:
                - c
                - d
            some_other_dict:
                another_key: more_data
                yet_another_key:
                    - this
                    - is
                    - getting
                    - silly
        can_bool: TRUE
...
"""
            stream = Modulemd.ModuleStream.read_string(yaml, True)

            assert stream is not None
            assert stream.props.module_name == "modulename"
            assert stream.props.stream_name == "streamname"
            assert stream.props.version == 1
            assert stream.props.context == "c0ffe3"
            assert stream.props.arch == "x86_64"
            assert stream.get_summary(locale="C") == "Module Summary"
            assert stream.get_description(locale="C") == "Module Description"

            assert "rpm_a" in stream.get_rpm_api()
            assert "rpm_b" in stream.get_rpm_api()

            assert "rpm_c" in stream.get_rpm_filters()

            assert (
                "bar-0:1.23-1.module_deadbeef.x86_64"
                in stream.get_rpm_artifacts()
            )

            assert "rawhide" in stream.get_servicelevel_names()
            assert "production" in stream.get_servicelevel_names()

            sl = stream.get_servicelevel("rawhide")
            assert sl is not None
            assert sl.props.name == "rawhide"
            assert sl.get_eol_as_string() == "2033-08-04"

            sl = stream.get_servicelevel("foo")
            assert sl is not None
            assert sl.props.name == "foo"
            assert sl.get_eol() is None

            sl = stream.get_servicelevel("production")
            assert sl is not None
            assert sl.props.name == "production"
            assert sl.get_eol() is not None
            assert sl.get_eol_as_string() == "2099-12-31"

            assert "DUMMY1" in stream.get_content_licenses()
            assert "DUMMY2" in stream.get_content_licenses()

            buildrequires = stream.get_buildtime_modules()
            assert len(buildrequires) == 2
            assert "platform" in buildrequires
            assert (
                stream.get_buildtime_requirement_stream("platform")
                == "and-its-stream-name"
            )
            assert "extra-build-env" in buildrequires
            assert (
                stream.get_buildtime_requirement_stream("extra-build-env")
                == "and-its-stream-name-too"
            )

            requires = stream.get_runtime_modules()
            assert len(requires) == 1
            assert "runtimeplatform" in requires
            assert (
                stream.get_runtime_requirement_stream("runtimeplatform")
                == "and-its-stream-name-2"
            )

            assert stream.props.community == "http://www.example.com/"
            assert stream.props.documentation == "http://www.example.com/"
            assert stream.props.tracker == "http://www.example.com/"

            assert len(stream.get_profile_names()) == 5

            buildopts = stream.get_buildopts()
            assert buildopts is not None

            assert (
                "%demomacro 1\n%demomacro2 %{demomacro}23\n"
                == buildopts.props.rpm_macros
            )
            assert "fooscl-1-bar" in buildopts.get_rpm_whitelist()
            assert "fooscl-1-baz" in buildopts.get_rpm_whitelist()
            assert "xxx" in buildopts.get_rpm_whitelist()
            assert "xyz" in buildopts.get_rpm_whitelist()

            if os.getenv("MMD_TEST_INSTALLED_LIB"):
                # The XMD python tests can only be run against the installed
                # lib because the overrides that translate between python and
                # GVariant must be installed in
                # /usr/lib/python*/site-packages/gi/overrides
                # or they are not included when importing Modulemd
                xmd = stream.get_xmd()
                assert xmd is not None

                assert "some_key" in xmd
                assert xmd["some_key"] == "some_data"

                assert "some_list" in xmd

                assert "a" in xmd["some_list"]
                assert "b" in xmd["some_list"]

                assert "some_dict" in xmd
                assert "a" in xmd["some_dict"]
                assert xmd["some_dict"]["a"] == "alpha"

                assert "some_other_dict" in xmd["some_dict"]
                assert "yet_another_key" in xmd["some_dict"]["some_other_dict"]
                assert (
                    "silly"
                    in xmd["some_dict"]["some_other_dict"]["yet_another_key"]
                )

                assert "can_bool" in xmd
                assert xmd["can_bool"] is True

            # Validate a trivial modulemd
            trivial_yaml = """
---
document: modulemd
version: 1
data:
  summary: Trivial Summary
  description: >-
    Trivial Description
  license:
    module: DUMMY
...
"""

            stream = Modulemd.ModuleStream.read_string(trivial_yaml, True)
            assert stream

            # Sanity check of spec.v1.yaml
            stream = Modulemd.ModuleStream.read_file(
                "%s/yaml_specs/modulemd_stream_v1.yaml"
                % os.getenv("MESON_SOURCE_ROOT"),
                True,
            )
            assert stream

    def test_packager_sanity(self):
        stream = Modulemd.ModuleStream.read_file(
            "%s/yaml_specs/modulemd_packager_v2.yaml"
            % os.getenv("MESON_SOURCE_ROOT"),
            True,
        )
        assert stream

    def test_depends_on_stream(self):

        for version in modulestream_versions:
            stream = Modulemd.ModuleStream.read_file(
                "%s/dependson_v%d.yaml"
                % (os.getenv("TEST_DATA_PATH"), version),
                True,
            )
            self.assertIsNotNone(stream)

            self.assertEqual(stream.depends_on_stream("platform", "f30"), True)
            self.assertEqual(
                stream.build_depends_on_stream("platform", "f30"), True
            )

            self.assertEqual(
                stream.depends_on_stream("platform", "f28"), False
            )
            self.assertEqual(
                stream.build_depends_on_stream("platform", "f28"), False
            )

            self.assertEqual(stream.depends_on_stream("base", "f30"), False)
            self.assertEqual(
                stream.build_depends_on_stream("base", "f30"), False
            )

            if version >= Modulemd.ModuleStreamVersionEnum.TWO:
                self.assertEqual(
                    stream.depends_on_stream("streamname", "f30"), True
                )
                self.assertEqual(
                    stream.build_depends_on_stream("streamname", "f30"), True
                )

    def test_validate_buildafter(self):
        for version in modulestream_versions:
            # buildafter is supported starting with v2
            if version < Modulemd.ModuleStreamVersionEnum.TWO:
                continue
            # Test a valid module stream with buildafter set
            stream = Modulemd.ModuleStream.read_file(
                "%s/buildafter/good_buildafter_v%d.yaml"
                % (self.test_data_path, version),
                True,
            )
            self.assertIsNotNone(stream)
            self.assertTrue(stream.validate())

            # Should fail validation if both buildorder and buildafter are set for
            # the same component.
            with self.assertRaisesRegex(
                gi.repository.GLib.GError,
                "Cannot mix buildorder and buildafter",
            ):
                stream = Modulemd.ModuleStream.read_file(
                    "%s/buildafter/both_same_component_v%d.yaml"
                    % (self.test_data_path, version),
                    True,
                )

            # Should fail validation if both buildorder and buildafter are set in
            # different components of the same stream.
            with self.assertRaisesRegex(
                gi.repository.GLib.GError,
                "Cannot mix buildorder and buildafter",
            ):
                stream = Modulemd.ModuleStream.read_file(
                    "%s/buildafter/mixed_buildorder_v%d.yaml"
                    % (self.test_data_path, version),
                    True,
                )

            # Should fail if a key specified in a buildafter set does not exist
            # for this module stream.
            with self.assertRaisesRegex(
                gi.repository.GLib.GError, "not found in components list"
            ):
                stream = Modulemd.ModuleStream.read_file(
                    "%s/buildafter/invalid_key_v%d.yaml"
                    % (self.test_data_path, version),
                    True,
                )

    def test_unicode_desc(self):
        for version in modulestream_versions:
            # Test a valid module stream with unicode in the description
            stream = Modulemd.ModuleStream.read_file(
                "%s/stream_unicode_v%d.yaml"
                % (os.getenv("TEST_DATA_PATH"), version),
                True,
                "",
                "",
            )

            self.assertIsNotNone(stream)
            self.assertTrue(stream.validate())

    def test_xmd_issue_274(self):
        # Test a valid module stream with unicode in the description
        stream = Modulemd.ModuleStream.read_file(
            "%s/stream_unicode_v2.yaml" % (os.getenv("TEST_DATA_PATH")),
            True,
            "",
            "",
        )

        # In this bug, we were getting a traceback attemping to call
        # get_xmd() more than once on the same stream. There were subtle
        # memory issues at play here.
        if "_overrides_module" in dir(Modulemd):
            # The XMD python tests can only be run against the installed lib
            # because the overrides that translate between python and GVariant
            # must be installed in /usr/lib/python*/site-packages/gi/overrides
            # or they are not included when importing Modulemd

            xmd = stream.get_xmd()
            mbs_xmd = stream.get_xmd()["mbs"]
            mbs_xmd2 = stream.get_xmd()["mbs"]

        else:
            stream.get_xmd()
            stream.get_xmd()

    def test_xmd_issue_290(self):
        if "_overrides_module" in dir(Modulemd):
            stream = Modulemd.ModuleStream.read_file(
                "%s/290.yaml" % (os.getenv("TEST_DATA_PATH")), True, "", ""
            )

            self.assertIsNotNone(stream)

            xmd = stream.get_xmd()

            xmd["something"] = ["foo", "bar"]
            stream.set_xmd(xmd)

            index = Modulemd.ModuleIndex()
            index.add_module_stream(stream)
            self.maxDiff = None
            output_yaml = index.dump_to_string()
            self.assertIsNotNone(output_yaml)
        pass

    def test_search_profiles(self):
        stream = Modulemd.ModuleStreamV2.new("themodule", "thestream")

        # First with no profiles added. Make sure we get back a zero-length
        # array

        profiles = stream.search_profiles(None)
        self.assertIsNotNone(profiles)
        self.assertEqual(len(profiles), 0)

        profiles = stream.search_profiles("*")
        self.assertIsNotNone(profiles)
        self.assertEqual(len(profiles), 0)

        profiles = stream.search_profiles("thefirstprofile")
        self.assertIsNotNone(profiles)
        self.assertEqual(len(profiles), 0)

        # Now add three profiles, and confirm that searches are
        # returned in alphabetical order
        stream.add_profile(Modulemd.Profile.new("thesecondprofile"))
        stream.add_profile(Modulemd.Profile.new("thefirstprofile"))
        stream.add_profile(Modulemd.Profile.new("thethirdprofile"))

        profiles = stream.search_profiles(None)
        self.assertIsNotNone(profiles)
        self.assertEqual(len(profiles), 3)
        self.assertIsInstance(profiles[0], Modulemd.Profile)
        self.assertEqual(profiles[0].get_name(), "thefirstprofile")
        self.assertIsInstance(profiles[1], Modulemd.Profile)
        self.assertEqual(profiles[1].get_name(), "thesecondprofile")
        self.assertIsInstance(profiles[2], Modulemd.Profile)
        self.assertEqual(profiles[2].get_name(), "thethirdprofile")

        profiles = stream.search_profiles("*")
        self.assertIsNotNone(profiles)
        self.assertEqual(len(profiles), 3)
        self.assertIsInstance(profiles[0], Modulemd.Profile)
        self.assertEqual(profiles[0].get_name(), "thefirstprofile")
        self.assertIsInstance(profiles[1], Modulemd.Profile)
        self.assertEqual(profiles[1].get_name(), "thesecondprofile")
        self.assertIsInstance(profiles[2], Modulemd.Profile)
        self.assertEqual(profiles[2].get_name(), "thethirdprofile")

        profiles = stream.search_profiles("thefirstprofile")
        self.assertIsNotNone(profiles)
        self.assertEqual(len(profiles), 1)
        self.assertIsInstance(profiles[0], Modulemd.Profile)
        self.assertEqual(profiles[0].get_name(), "thefirstprofile")

        profiles = stream.search_profiles("*profile")
        self.assertIsNotNone(profiles)
        self.assertEqual(len(profiles), 3)
        self.assertIsInstance(profiles[0], Modulemd.Profile)
        self.assertEqual(profiles[0].get_name(), "thefirstprofile")
        self.assertIsInstance(profiles[1], Modulemd.Profile)
        self.assertEqual(profiles[1].get_name(), "thesecondprofile")
        self.assertIsInstance(profiles[2], Modulemd.Profile)
        self.assertEqual(profiles[2].get_name(), "thethirdprofile")

        profiles = stream.search_profiles("*dprofile*")
        self.assertIsNotNone(profiles)
        self.assertEqual(len(profiles), 2)
        self.assertIsInstance(profiles[0], Modulemd.Profile)
        self.assertEqual(profiles[0].get_name(), "thesecondprofile")
        self.assertIsInstance(profiles[1], Modulemd.Profile)
        self.assertEqual(profiles[1].get_name(), "thethirdprofile")

        profiles = stream.search_profiles("the*profile")
        self.assertIsNotNone(profiles)
        self.assertEqual(len(profiles), 3)
        self.assertIsInstance(profiles[0], Modulemd.Profile)
        self.assertEqual(profiles[0].get_name(), "thefirstprofile")
        self.assertIsInstance(profiles[1], Modulemd.Profile)
        self.assertEqual(profiles[1].get_name(), "thesecondprofile")
        self.assertIsInstance(profiles[2], Modulemd.Profile)
        self.assertEqual(profiles[2].get_name(), "thethirdprofile")

        profiles = stream.search_profiles("the*")
        self.assertIsNotNone(profiles)
        self.assertEqual(len(profiles), 3)
        self.assertIsInstance(profiles[0], Modulemd.Profile)
        self.assertEqual(profiles[0].get_name(), "thefirstprofile")
        self.assertIsInstance(profiles[1], Modulemd.Profile)
        self.assertEqual(profiles[1].get_name(), "thesecondprofile")
        self.assertIsInstance(profiles[2], Modulemd.Profile)
        self.assertEqual(profiles[2].get_name(), "thethirdprofile")

    def test_static_context(self):
        stream = Modulemd.ModuleStreamV2.new("themodule", "thestream")

        # Verify that it defaults to FALSE
        self.assertFalse(stream.props.static_context)
        self.assertFalse(stream.is_static_context())

        # Test setting and unsetting it via methods
        stream.set_static_context()
        self.assertTrue(stream.props.static_context)
        self.assertTrue(stream.is_static_context())

        stream.unset_static_context()
        self.assertFalse(stream.props.static_context)
        self.assertFalse(stream.is_static_context())

        # Test setting and unsetting it via properties
        stream.props.static_context = True
        self.assertTrue(stream.props.static_context)
        self.assertTrue(stream.is_static_context())

        stream.props.static_context = False
        self.assertFalse(stream.props.static_context)
        self.assertFalse(stream.is_static_context())

        # Read in a stream with a static context
        idx = Modulemd.load_file(
            "%s/static_context.yaml" % (os.getenv("TEST_DATA_PATH"))
        )
        streams = idx.search_streams()
        self.assertEqual(1, len(streams))
        stream = streams[0]
        self.assertTrue(stream.props.static_context)
        self.assertTrue(stream.is_static_context())

        expected = """---
document: modulemd
version: 2
data:
  name: nodejs
  stream: "8"
  version: 20180816123422
  context: RealCTX
  static_context: true
  arch: x86_64
  summary: Javascript runtime
  description: >-
    Node.js is a platform built on Chrome''s JavaScript runtime for easily building
    fast, scalable network applications. Node.js uses an event-driven, non-blocking
    I/O model that makes it lightweight and efficient, perfect for data-intensive
    real-time applications that run across distributed devices.
  license:
    module:
    - MIT
    content:
    - DUMMY
  dependencies:
  - buildrequires:
      platform: [f29]
    requires:
      platform: [f29]
  references:
    community: http://nodejs.org
    documentation: http://nodejs.org/en/docs
    tracker: https://github.com/nodejs/node/issues
  profiles:
    default:
      rpms:
      - nodejs
      - npm
    development:
      rpms:
      - nodejs
      - nodejs-devel
      - npm
    minimal:
      rpms:
      - nodejs
  api:
    rpms:
    - nodejs
    - nodejs-devel
    - npm
  components:
    rpms:
      nodejs:
        rationale: Javascript runtime and npm package manager.
        repository: git://pkgs.fedoraproject.org/rpms/nodejs
        cache: http://pkgs.fedoraproject.org/repo/pkgs/nodejs
        ref: "8"
        buildorder: 10
  artifacts:
    rpms:
    - nodejs-1:8.11.4-1.module_2030+42747d40.x86_64
    - nodejs-devel-1:8.11.4-1.module_2030+42747d40.x86_64
    - nodejs-docs-1:8.11.4-1.module_2030+42747d40.noarch
    - npm-1:5.6.0-1.8.11.4.1.module_2030+42747d40.x86_64
...
"""
        self.maxDiff = None
        self.assertEqual(expected, idx.dump_to_string())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# This file is part of libmodulemd
# Copyright (C) 2018 Red Hat, Inc.
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


class TestTranslationEntry(TestBase):
    def test_constructors(self):
        # Test that the new() function works
        te = Modulemd.TranslationEntry.new("en_US")
        assert te
        assert te.props.locale == "en_US"
        assert te.get_locale() == "en_US"
        assert te.props.summary is None
        assert te.get_summary() is None
        assert te.props.description is None
        assert te.get_description() is None
        assert te.get_profiles() == []
        assert te.get_profile_description("test") is None

        # Test that keyword arg locale is accepted
        te = Modulemd.TranslationEntry(locale="en_US")
        assert te
        assert te.props.locale == "en_US"
        assert te.get_locale() == "en_US"
        assert te.props.summary is None
        assert te.get_summary() is None
        assert te.props.description is None
        assert te.get_description() is None
        assert te.get_profiles() == []
        assert te.get_profile_description("test") is None

        # Test that init works with locale and summary
        te = Modulemd.TranslationEntry(locale="en_US", summary="foobar")
        assert te
        assert te.props.locale == "en_US"
        assert te.get_locale() == "en_US"
        assert te.props.summary == "foobar"
        assert te.get_summary() == "foobar"
        assert te.props.description is None
        assert te.get_description() is None
        assert te.get_profiles() == []
        assert te.get_profile_description("test") is None

        # Test that init works with locale and description
        te = Modulemd.TranslationEntry(locale="en_US", description="barfoo")
        assert te
        assert te.props.locale == "en_US"
        assert te.get_locale() == "en_US"
        assert te.props.summary is None
        assert te.get_summary() is None
        assert te.props.description == "barfoo"
        assert te.get_description() == "barfoo"
        assert te.get_profiles() == []
        assert te.get_profile_description("test") is None

        # Test that init works with locale, summary and description
        te = Modulemd.TranslationEntry(
            locale="en_US", summary="foobar", description="barfoo"
        )
        assert te
        assert te.props.locale == "en_US"
        assert te.get_locale() == "en_US"
        assert te.props.summary == "foobar"
        assert te.get_summary() == "foobar"
        assert te.props.description == "barfoo"
        assert te.get_description() == "barfoo"
        assert te.get_profiles() == []
        assert te.get_profile_description("test") is None

        # Test that init works with locale, unicode summary and unicode
        # description
        te = Modulemd.TranslationEntry(
            locale="ro_TA",  # robots_Tables
            summary="(┛ಠ_ಠ)┛彡┻━┻",
            description="(┛ಠ_ಠ)┛彡",
        )
        assert te
        assert te.props.locale == "ro_TA"
        assert te.get_locale() == "ro_TA"
        assert te.props.summary == "(┛ಠ_ಠ)┛彡┻━┻"
        assert te.get_summary() == "(┛ಠ_ಠ)┛彡┻━┻"
        assert te.props.description == "(┛ಠ_ಠ)┛彡"
        assert te.get_description() == "(┛ಠ_ಠ)┛彡"
        assert te.get_profiles() == []
        assert te.get_profile_description("test") is None

        # Test that we fail if we call new() with a None locale
        try:
            te = Modulemd.TranslationEntry.new(None)
            assert False
        except TypeError as e:
            assert "does not allow None as a value" in e.__str__()

        # Test that we fail if object is instantiated without a locale
        with self.expect_signal():
            Modulemd.TranslationEntry()

        # Test that we fail if object is instantiated with a None locale
        with self.expect_signal():
            Modulemd.TranslationEntry(locale=None)

    def test_copy(self):
        te_orig = Modulemd.TranslationEntry(locale="en_US")
        te = te_orig.copy()
        assert te
        assert te.props.locale == "en_US"
        assert te.get_locale() == "en_US"
        assert te.props.summary is None
        assert te.get_summary() is None
        assert te.props.description is None
        assert te.get_description() is None
        assert te.get_profiles() == []
        assert te.get_profile_description("test") is None

        te_orig.set_summary("foobar")
        te_orig.set_description("barfoo")
        te_orig.set_profile_description("test1", "brown fox")
        te_orig.set_profile_description("test2", "jumped")

        te = te_orig.copy()
        assert te
        assert te.props.locale == "en_US"
        assert te.get_locale() == "en_US"
        assert te.props.summary == "foobar"
        assert te.get_summary() == "foobar"
        assert te.props.description == "barfoo"
        assert te.get_description() == "barfoo"
        assert te.get_profiles() == ["test1", "test2"]
        assert te.get_profile_description("test") is None
        assert te.get_profile_description("test1") == "brown fox"
        assert te.get_profile_description("test2") == "jumped"

    def test_get_locale(self):
        te = Modulemd.TranslationEntry(locale="en_US")

        assert te.get_locale() == "en_US"
        assert te.props.locale == "en_US"

        with self.expect_signal():
            te.props.locale = "en_GB"

    def test_get_set_summary(self):
        te = Modulemd.TranslationEntry(locale="en_US")

        assert te.props.locale == "en_US"
        assert te.get_locale() == "en_US"
        assert te.props.summary is None
        assert te.get_summary() is None

        te.set_summary("foobar")
        assert te.props.summary == "foobar"
        assert te.get_summary() == "foobar"

        te.props.summary = "barfoo"
        assert te.props.summary == "barfoo"
        assert te.get_summary() == "barfoo"

        te.props.summary = None
        assert te.props.summary is None
        assert te.get_summary() is None

    def test_get_set_description(self):
        te = Modulemd.TranslationEntry(locale="en_US")

        assert te.props.locale == "en_US"
        assert te.get_locale() == "en_US"
        assert te.props.description is None
        assert te.get_description() is None

        te.set_description("foobar")
        assert te.props.description == "foobar"
        assert te.get_description() == "foobar"

        te.props.description = "barfoo"
        assert te.props.description == "barfoo"
        assert te.get_description() == "barfoo"

        te.props.description = None
        assert te.props.description is None
        assert te.get_description() is None

    def test_profile_descriptions(self):
        te = Modulemd.TranslationEntry(locale="en_US")

        assert te.props.locale == "en_US"
        assert te.get_locale() == "en_US"
        assert te.get_profiles() == []
        assert te.get_profile_description("test1") is None
        assert te.get_profile_description("test2") is None
        assert te.get_profile_description("test3") is None

        # Add a profile
        te.set_profile_description("test1", "foobar")
        assert te.get_profiles() == ["test1"]
        assert te.get_profile_description("test1") == "foobar"
        assert te.get_profile_description("test2") is None
        assert te.get_profile_description("test3") is None

        # Add a second profile
        te.set_profile_description("test2", "barfoo")
        assert te.get_profiles() == ["test1", "test2"]
        assert te.get_profile_description("test1") == "foobar"
        assert te.get_profile_description("test2") == "barfoo"
        assert te.get_profile_description("test3") is None

        # Add a third one that is supposed to go before the others
        te.set_profile_description("test3", "foobarfoo")
        assert te.get_profiles() == ["test1", "test2", "test3"]
        assert te.get_profile_description("test1") == "foobar"
        assert te.get_profile_description("test2") == "barfoo"
        assert te.get_profile_description("test3") == "foobarfoo"


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# This file is part of libmodulemd
# Copyright (C) 2020 Stephen Gallagher
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import xmlrpc.client
import time

KOJI_URL = "https://koji.fedoraproject.org/kojihub"


def get_fedora_rawhide_version(session):
    # Koji sometimes disconnects for no apparent reason. Retry up to 5
    # times before failing.
    for attempt in range(5):
        try:
            build_targets = session.getBuildTargets("rawhide")
        except requests.exceptions.ConnectionError:
            logging.warning(
                "Connection lost while retrieving rawhide branch, retrying..."
            )
        else:
            # Succeeded this time, so break out of the loop
            break
        time.sleep(3)

    return build_targets[0]["build_tag_name"].partition("-build")[0][1:]


def main():
    session = xmlrpc.client.ServerProxy(KOJI_URL)
    print(get_fedora_rawhide_version(session))


if __name__ == "__main__":
    main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2017-2018 Stephen Gallagher
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.


import os
import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import GLib
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


class TestRpmMapEntry(TestBase):
    def test_basic(self):
        # Test that the new() function works
        entry = Modulemd.RpmMapEntry.new(
            "bar", 0, "1.23", "1.module_deadbeef", "x86_64"
        )

        self.assertIsNotNone(entry)

        self.assertEqual(entry.props.name, "bar")
        self.assertEqual(entry.props.epoch, 0)
        self.assertEqual(entry.props.version, "1.23")
        self.assertEqual(entry.props.release, "1.module_deadbeef")
        self.assertEqual(entry.props.arch, "x86_64")
        self.assertEqual(
            entry.props.nevra, "bar-0:1.23-1.module_deadbeef.x86_64"
        )

        # Test that object instantiation with attributes works
        entry2 = Modulemd.RpmMapEntry(
            name="bar",
            version="1.23",
            release="1.module_deadbeef",
            arch="x86_64",
        )
        self.assertEqual(
            entry2.props.nevra, "bar-0:1.23-1.module_deadbeef.x86_64"
        )

        # Test that nevra returns NULL if attributes are missing

        # Remove name
        entry2.props.name = None
        with self.assertRaisesRegex(
            gi.repository.GLib.GError, "Missing name attribute"
        ):
            entry2.validate()
        self.assertIsNone(entry2.props.nevra)
        entry2.props.name = "bar"

        # Remove the version
        entry2.props.version = None
        with self.assertRaisesRegex(
            gi.repository.GLib.GError, "Missing version attribute"
        ):
            entry2.validate()
        self.assertIsNone(entry2.props.nevra)
        entry2.props.version = "1.23"

        # Remove the release
        entry2.props.release = None
        with self.assertRaisesRegex(
            gi.repository.GLib.GError, "Missing release attribute"
        ):
            entry2.validate()
        self.assertIsNone(entry2.props.nevra)
        entry2.props.release = "1.module_deadbeef"

        # Remove the arch
        entry2.props.arch = None
        with self.assertRaisesRegex(
            gi.repository.GLib.GError, "Missing arch attribute"
        ):
            entry2.validate()
        self.assertIsNone(entry2.props.nevra)
        entry2.props.arch = "x86_64"

        self.assertEqual(
            entry2.props.nevra, "bar-0:1.23-1.module_deadbeef.x86_64"
        )

    def test_compare(self):
        entry = Modulemd.RpmMapEntry.new(
            "bar", 0, "1.23", "1.module_deadbeef", "x86_64"
        )
        self.assertIsNotNone(entry)

        entry2 = Modulemd.RpmMapEntry.new(
            "bar", 0, "1.23", "1.module_deadbeef", "x86_64"
        )
        self.assertIsNotNone(entry2)

        entry3 = Modulemd.RpmMapEntry.new(
            "foo", 0, "1.23", "1.module_deadbeef", "x86_64"
        )
        self.assertIsNotNone(entry3)

        self.assertTrue(entry.equals(entry))
        self.assertTrue(entry.equals(entry2))
        self.assertFalse(entry.equals(entry3))


if __name__ == "__main__":
    unittest.main()



import json
import logging
import os
import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import GLib
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase

logging.basicConfig(level=logging.DEBUG)


class TestCommon(TestBase):
    def test_packager_read_file(self):
        # We have a chicken-egg problem with overrides, since they can only
        # be tested if they are already installed. This means they need to
        # be run in the CI. In order to avoid changes to these tests or the
        # overrides breaking things, we'll skip them if the appropriate
        # override is not installed.
        if not (
            "_overrides_module" in dir(Modulemd)
            and hasattr(gi.overrides.Modulemd, "read_packager_file")
        ):
            logging.debug(
                "read_packager_file() override not installed, skipping tests"
            )
            return

        # Validate that the PackagerV2 specification parses correctly
        doc = Modulemd.read_packager_file(
            os.path.join(
                self.source_root, "yaml_specs/modulemd_packager_v2.yaml"
            )
        )
        self.assertIsNotNone(doc)
        # Once read in, a modulemd-packager v2 document is a ModuleStream of the
        # same version.
        self.assertIs(type(doc), Modulemd.ModuleStreamV2)
        # Confirm module and stream names are undefined
        self.assertIsNone(doc.get_module_name())
        self.assertIsNone(doc.get_stream_name())

        # Read the PackagerV2 specification with module/stream name overrides
        doc = Modulemd.read_packager_file(
            os.path.join(
                self.source_root, "yaml_specs/modulemd_packager_v2.yaml"
            ),
            "modulename-override",
            "streamname-override",
        )
        self.assertIsNotNone(doc)
        self.assertIs(type(doc), Modulemd.ModuleStreamV2)
        # Confirm module and stream names were set
        self.assertEqual(doc.get_module_name(), "modulename-override")
        self.assertEqual(doc.get_stream_name(), "streamname-override")

        # Valid PackagerV3 file
        doc = Modulemd.read_packager_file(
            os.path.join(
                self.source_root, "yaml_specs/modulemd_packager_v3.yaml"
            )
        )
        self.assertIsNotNone(doc)
        self.assertIs(type(doc), Modulemd.PackagerV3)

        # Read PackagerV3 with module/stream name overrides
        doc = Modulemd.read_packager_file(
            os.path.join(
                self.source_root, "yaml_specs/modulemd_packager_v3.yaml"
            ),
            "modulename-override",
            "streamname-override",
        )
        self.assertIsNotNone(doc)
        self.assertIs(type(doc), Modulemd.PackagerV3)
        # Confirm module and stream names were set
        self.assertEqual(doc.get_module_name(), "modulename-override")
        self.assertEqual(doc.get_stream_name(), "streamname-override")

        # Validate that the ModuleStreamV2 specification parses correctly
        doc = Modulemd.read_packager_file(
            os.path.join(
                self.source_root, "yaml_specs/modulemd_stream_v2.yaml"
            )
        )
        self.assertIsNotNone(doc)
        self.assertIs(type(doc), Modulemd.ModuleStreamV2)
        # Confirm module and stream names are correct
        self.assertEqual(doc.get_module_name(), "foo")
        self.assertEqual(doc.get_stream_name(), "latest")

        # Read the ModuleStreamV2 specification with module/stream name overrides
        doc = Modulemd.read_packager_file(
            os.path.join(
                self.source_root, "yaml_specs/modulemd_stream_v2.yaml"
            ),
            "modulename-override",
            "streamname-override",
        )
        self.assertIsNotNone(doc)
        self.assertIs(type(doc), Modulemd.ModuleStreamV2)
        # Confirm module and stream names were set
        self.assertEqual(doc.get_module_name(), "modulename-override")
        self.assertEqual(doc.get_stream_name(), "streamname-override")

    def test_packager_read_string(self):
        # We have a chicken-egg problem with overrides, since they can only
        # be tested if they are already installed. This means they need to
        # be run in the CI. In order to avoid changes to these tests or the
        # overrides breaking things, we'll skip them if the appropriate
        # override is not installed.
        if not (
            "_overrides_module" in dir(Modulemd)
            and hasattr(gi.overrides.Modulemd, "read_packager_string")
        ):
            logging.debug(
                "read_packager_string() override not installed, skipping tests"
            )
            return

        # PackagerV2
        minimal_valid = """---
document: modulemd-packager
version: 2
data:
  summary: A minimal valid module
  description: A minimalistic module description
  license:
    module:
      - DUMMY
...
"""
        doc = Modulemd.read_packager_string(minimal_valid)
        self.assertIsNotNone(doc)
        # Once read in, a modulemd-packager document is a ModuleStream of the
        # same version.
        self.assertIs(type(doc), Modulemd.ModuleStreamV2)
        # Confirm module and stream names are undefined
        self.assertIsNone(doc.get_module_name())
        self.assertIsNone(doc.get_stream_name())

        # Read again with module/stream name overrides
        doc = Modulemd.read_packager_string(
            minimal_valid, "modulename-override", "streamname-override"
        )
        self.assertIsNotNone(doc)
        self.assertIs(type(doc), Modulemd.ModuleStreamV2)
        # Confirm module and stream names were set
        self.assertEqual(doc.get_module_name(), "modulename-override")
        self.assertEqual(doc.get_stream_name(), "streamname-override")

        # PackagerV3
        minimal_valid = """---
document: modulemd-packager
version: 3
data:
  summary: A minimal valid module
  description: A minimalistic module description
  license:
    - DUMMY
...
"""
        doc = Modulemd.read_packager_string(minimal_valid)
        self.assertIsNotNone(doc)
        self.assertIs(type(doc), Modulemd.PackagerV3)
        # Confirm module and stream names are undefined
        self.assertIsNone(doc.get_module_name())
        self.assertIsNone(doc.get_stream_name())

        # Read again with module/stream name overrides
        doc = Modulemd.read_packager_string(
            minimal_valid, "modulename-override", "streamname-override"
        )
        self.assertIsNotNone(doc)
        self.assertIs(type(doc), Modulemd.PackagerV3)
        # Confirm module and stream names were set
        self.assertEqual(doc.get_module_name(), "modulename-override")
        self.assertEqual(doc.get_stream_name(), "streamname-override")

        # ModuleStreamV2
        minimal_valid = """---
document: modulemd-stream
version: 2
data:
  summary: A minimal valid module
  description: A minimalistic module description
  license:
    module:
      - DUMMY
...
"""
        doc = Modulemd.read_packager_string(minimal_valid)
        self.assertIsNotNone(doc)
        self.assertIs(type(doc), Modulemd.ModuleStreamV2)
        # Confirm module and stream names are undefined
        self.assertIsNone(doc.get_module_name())
        self.assertIsNone(doc.get_stream_name())

        # Read again with module/stream name overrides
        doc = Modulemd.read_packager_string(
            minimal_valid, "modulename-override", "streamname-override"
        )
        self.assertIsNotNone(doc)
        self.assertIs(type(doc), Modulemd.ModuleStreamV2)
        # Confirm module and stream names were set
        self.assertEqual(doc.get_module_name(), "modulename-override")
        self.assertEqual(doc.get_stream_name(), "streamname-override")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2018 Red Hat, Inc.
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


class TestDefaults(TestBase):
    def test_constructors(self):
        # Test that the new() function works
        defs = Modulemd.DefaultsV1.new("foo")
        assert defs
        assert defs.props.mdversion == Modulemd.DefaultsVersionEnum.ONE
        assert defs.get_mdversion() == Modulemd.DefaultsVersionEnum.ONE
        assert defs.props.module_name == "foo"
        assert defs.get_module_name() == "foo"

        # Test gobject instantiation
        defs = Modulemd.DefaultsV1(module_name="foo")
        assert defs
        assert defs.props.mdversion == Modulemd.DefaultsVersionEnum.ONE
        assert defs.get_mdversion() == Modulemd.DefaultsVersionEnum.ONE
        assert defs.props.module_name == "foo"
        assert defs.get_module_name() == "foo"

        # Test with no name
        with self.assertRaisesRegex(
            TypeError, "does not allow None as a value"
        ):
            defs = Modulemd.DefaultsV1.new(None)

    def test_copy(self):
        defs = Modulemd.DefaultsV1.new("foo")
        assert defs

        copied_defs = defs.copy()
        assert copied_defs
        assert copied_defs.props.mdversion == defs.props.mdversion
        assert copied_defs.props.module_name == defs.props.module_name

        # Add some actual intents and streams and compare the copy
        defs.set_default_stream("latest")
        defs.set_default_stream("latest", "server_intent")

        copied_defs = defs.copy()
        assert copied_defs
        assert copied_defs.props.mdversion == defs.props.mdversion
        assert copied_defs.props.module_name == defs.props.module_name
        assert copied_defs.get_default_stream() == defs.get_default_stream()
        assert copied_defs.get_default_stream(
            "server_intent"
        ) == defs.get_default_stream("server_intent")

    def test_get_set_default_stream(self):
        defs = Modulemd.DefaultsV1.new("foo")
        assert defs
        defs.set_default_stream("latest")
        assert defs.get_default_stream() == "latest"

        defs.set_default_stream("experimental", "workstation_intent")
        assert defs.get_default_stream("workstation_intent") == "experimental"

        defs.set_default_stream(None)
        assert defs.get_default_stream() is None

        defs.set_default_stream(None, "minimal_intent")
        assert defs.get_default_stream("minimal_intent") is None

    def test_get_set_profiles(self):
        defs = Modulemd.DefaultsV1.new("foo")

        defs.add_default_profile_for_stream("latest", "client")
        defs.add_default_profile_for_stream("latest", "server")
        defs.add_default_profile_for_stream(
            "latest", "server", "server_intent"
        )

        assert "client" in defs.get_default_profiles_for_stream("latest")
        assert "client" in defs.get_default_profiles_for_stream("latest")

    def test_validate(self):
        defs = Modulemd.Defaults.new(
            Modulemd.DefaultsVersionEnum.LATEST, "foo"
        )
        assert defs

        assert defs.validate()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2018 Red Hat, Inc.
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase


class TestTranslation(TestBase):
    def test_constructors(self):
        # Test that the new() function works
        t = Modulemd.Translation.new(1, "testmodule", "teststream", 42)
        assert t

        # Test that keywords are accepted
        t = Modulemd.Translation(
            version=1,
            module_name="testmodule",
            module_stream="teststream",
            modified=42,
        )
        assert t
        assert t.validate()
        assert t.get_locales() == []

    def test_copy(self):
        t_orig = Modulemd.Translation.new(1, "testmodule", "teststream", 42)
        t = t_orig.copy()
        assert t
        assert t.validate()
        assert t.get_locales() == []


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

# This file is part of libmodulemd
# Copyright (C) 2017-2018 Stephen Gallagher
#
# Fedora-License-Identifier: MIT
# SPDX-2.0-License-Identifier: MIT
# SPDX-3.0-License-Identifier: MIT
#
# This program is free software.
# For more information on the license, see COPYING.
# For more information on free software, see
# <https://www.gnu.org/philosophy/free-sw.en.html>.

import sys

try:
    import unittest
    import gi

    gi.require_version("Modulemd", "2.0")
    from gi.repository import Modulemd
    from gi.repository import GLib
except ImportError:
    # Return error 77 to skip this test on platforms without the necessary
    # python modules
    sys.exit(77)

from base import TestBase
import datetime


class TestServiceLevel(TestBase):
    def test_constructors(self):
        # Test that the new() function works
        sl = Modulemd.ServiceLevel.new("foo")
        assert sl
        assert sl.props.name == "foo"
        assert sl.get_name() == "foo"
        assert sl.get_eol() is None
        assert sl.get_eol_as_string() is None

        # Test that standard object instatiation works with a name
        sl = Modulemd.ServiceLevel(name="foo")
        assert sl
        assert sl.props.name == "foo"
        assert sl.get_name() == "foo"
        assert sl.get_eol() is None
        assert sl.get_eol_as_string() is None

        # Test that we fail if we call new() with a None name
        try:
            sl = Modulemd.ServiceLevel.new(None)
            assert False
        except TypeError as e:
            assert "does not allow None as a value" in e.__str__()

        # Test that we fail if object is instantiated without a name
        with self.expect_signal():
            sl = Modulemd.ServiceLevel()

        # Test that we fail if object is instantiated with a None name
        with self.expect_signal():
            sl = Modulemd.ServiceLevel(name=None)

    def test_copy(self):
        sl = Modulemd.ServiceLevel.new("foo")
        assert sl
        assert sl.props.name == "foo"
        assert sl.get_name() == "foo"
        assert sl.get_eol() is None
        assert sl.get_eol_as_string() is None

        sl_copy = sl.copy()
        assert sl_copy
        assert sl_copy.props.name == "foo"
        assert sl_copy.get_name() == "foo"
        assert sl_copy.get_eol() is None
        assert sl_copy.get_eol_as_string() is None

        sl.set_eol_ymd(2018, 11, 13)
        sl_copy = sl.copy()
        assert sl_copy
        assert sl_copy.props.name == "foo"
        assert sl_copy.get_name() == "foo"
        assert sl_copy.get_eol() is not None
        assert sl_copy.get_eol_as_string() == "2018-11-13"

    def test_get_name(self):
        sl = Modulemd.ServiceLevel.new("foo")

        assert sl.get_name() == "foo"
        assert sl.props.name == "foo"

        # This property is not writable, make sure it fails to attempt it
        with self.expect_signal():
            sl.props.name = "bar"

    def test_get_set_eol(self):
        sl = Modulemd.ServiceLevel.new("foo")

        if "_overrides_module" in dir(Modulemd):
            # Test that EOL is initialized to None
            assert sl.get_eol() is None

            # Test the set_eol() method
            eol = datetime.date(2018, 11, 7)
            sl.set_eol(eol)

            returned_eol = sl.get_eol()
            assert returned_eol is not None
            assert returned_eol == eol
            assert sl.get_eol_as_string() == "2018-11-07"

            # Test the set_eol_ymd() method
            sl.set_eol_ymd(2019, 12, 3)

            returned_eol = sl.get_eol()
            assert returned_eol is not None
            assert returned_eol.day == 3
            assert returned_eol.month == 12
            assert returned_eol.year == 2019
            assert sl.get_eol_as_string() == "2019-12-03"

            # There is no February 31
            sl.set_eol_ymd(2011, 2, 31)
            assert sl.get_eol() is None

        else:
            # Test that EOL is initialized to None
            assert sl.get_eol() is None

            # Test the set_eol() method
            eol = GLib.Date.new_dmy(7, 11, 2018)
            sl.set_eol(eol)

            returned_eol = sl.get_eol()
            assert returned_eol is not None
            assert returned_eol.get_day() == eol.get_day()
            assert returned_eol.get_month() == eol.get_month()
            assert returned_eol.get_year() == eol.get_year()
            assert sl.get_eol_as_string() == "2018-11-07"

            # Test the set_eol_ymd() method
            sl.set_eol_ymd(2019, 12, 3)

            returned_eol = sl.get_eol()
            assert returned_eol is not None
            assert returned_eol.get_day() == 3
            assert returned_eol.get_month() == 12
            assert returned_eol.get_year() == 2019
            assert sl.get_eol_as_string() == "2019-12-03"

            # Try setting some invalid dates
            # An initialized but unset date
            eol = GLib.Date.new()
            sl.set_eol(eol)
            assert sl.get_eol() is None

            # There is no February 31
            sl.set_eol_ymd(2011, 2, 31)
            assert sl.get_eol() is None


if __name__ == "__main__":
    unittest.main()
